import logging
from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from django.utils.decorators import method_decorator
from django.utils.translation import gettext as _
from django.views.generic import View
from django.views.decorators.csrf import csrf_exempt
from django.utils.translation import get_language, get_language_bidi
from arches.app.models.tile import Tile
from arches.app.models.resource import Resource
from arches.app.models.models import ResourceXResource
from arches.app.utils.betterJSONSerializer import JSONDeserializer
from arches.app.utils.response import JSONResponse

logger = logging.getLogger(__name__)


def get_related_resource_template(resourceid, relationship_type="", inverse_relationship_type=""):
    return {
        "resourceId": resourceid,
        "ontologyProperty": relationship_type,
        "resourceXresourceId": "",
        "inverseOntologyProperty": inverse_relationship_type,
    }


class SaveAnnotationView(View):
    def create_physical_thing_resource(self, request, transaction_id):
        physical_thing_graphid = "9519cb4f-b25b-11e9-8c7b-a4d18cec433a"
        resource = Resource()
        resource.graph_id = physical_thing_graphid
        resource.save(transaction_id=transaction_id, request=request)
        resourceid = str(resource.pk)

        return resourceid

    def save_node(self, request, resourceinstanceid, nodegroupid, nodeid, transactionid, nodevalue, tileid=None):
        if tileid is not None:
            tile = Tile.objects.get(pk=tileid)
        else:
            try:
                tile = Tile.objects.get(resourceinstance=resourceinstanceid, nodegroup_id=nodegroupid)
            except ObjectDoesNotExist as e:
                tile = Tile.get_blank_tile(nodeid=nodeid, resourceid=resourceinstanceid)
        tile.data[nodeid] = nodevalue
        tile.save(transaction_id=transactionid, request=request, index=False)

        return tile

    def save_tile(self, request, resourceinstanceid, nodegroupid, transactionid, tiledata, tileid=None):
        if tileid is not None:
            tile = Tile.objects.get(pk=tileid)
        else:
            tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=nodegroupid, resourceid=resourceinstanceid)
        tile.data = tiledata
        tile.save(transaction_id=transactionid, request=request, index=False)

        return tile

    def save_related_resource_node(
        self,
        request,
        resourceinstanceid,
        nodeid,
        transactionid,
        related_resourceid,
        tileid=None,
        relationship_type="",
        inverse_relationship_type="",
    ):
        if tileid is not None:
            tile = Tile.objects.get(pk=tileid)
            tile.data[nodeid][0]["resourceId"] = related_resourceid
        else:
            try:
                tile = Tile.objects.get(resourceinstance=resourceinstanceid, nodegroup_id=nodeid)
            except ObjectDoesNotExist as e:
                tile = Tile.get_blank_tile(nodeid=nodeid, resourceid=resourceinstanceid)
            related_resource_template = get_related_resource_template(related_resourceid, relationship_type, inverse_relationship_type)
            tile.data[nodeid] = [related_resource_template]
        tile.save(transaction_id=transactionid, request=request, index=False)

        return tile

    def add_related_resource_node(self, request, resourceinstanceid, nodeid, transactionid, related_resourceid, tileid=None):
        if tileid is not None:
            tile = Tile.objects.get(pk=tileid)
            tile.data[nodeid](0)["resourceId"] = related_resourceid
            # do I want to update resource x resource data?
        else:
            tile = Tile.get_blank_tile(nodeid=nodeid, resourceid=resourceinstanceid)
            related_resource_template = get_related_resource_template(related_resourceid)
            tile.data[nodeid] = [related_resource_template]
        tile.save(transaction_id=transactionid, request=request, index=False)

        return tile

    def add_to_related_resource_list_tile(self, request, resourceinstanceid, nodeid, transactionid, related_resourceid, tileid=None):
        if tileid is not None:
            tile = Tile.objects.get(pk=tileid)
        else:
            try:
                tile = Tile.objects.get(resourceinstance=resourceinstanceid, nodegroup_id=nodeid)
            except ObjectDoesNotExist as e:
                tile = Tile.get_blank_tile(nodeid=nodeid, resourceid=resourceinstanceid)
                tile.data[nodeid] = []

        list_of_rr_resources = [data["resourceId"] for data in tile.data[nodeid]]
        if related_resourceid not in list_of_rr_resources:
            related_resource_template = get_related_resource_template(related_resourceid)
            tile.data[nodeid].append(related_resource_template)
            tile.save(transaction_id=transactionid, request=request, index=False)

        return tile

    def save_physical_thing_name(self, request, resourceid, transactionid, name, tileid=None):
        physical_thing_name_nodegroupid = "b9c1ced7-b497-11e9-a4da-a4d18cec433a"
        physical_thing_name_nodeid = "b9c1d8a6-b497-11e9-876b-a4d18cec433a"
        physical_thing_name_type_nodeid = "b9c1d7ab-b497-11e9-9ab7-a4d18cec433a"
        physical_thing_name_language_nodeid = "b9c1d400-b497-11e9-90ea-a4d18cec433a"
        preferred_terms_conceptid = "8f40c740-3c02-4839-b1a4-f1460823a9fe"
        english_conceptid = "bc35776b-996f-4fc1-bd25-9f6432c1f349"

        if tileid is not None:
            tile = Tile.objects.get(pk=tileid)
        else:
            try:
                tile = Tile.objects.get(resourceinstance=resourceid, nodegroup_id=physical_thing_name_nodegroupid)
            except ObjectDoesNotExist as e:
                tile = Tile.get_blank_tile(nodeid=physical_thing_name_nodeid, resourceid=resourceid)
        tile.data[physical_thing_name_nodeid] = {}
        tile.data[physical_thing_name_nodeid][get_language()] = {"value": name, "direction": "rtl" if get_language_bidi() else "ltr"}
        tile.data[physical_thing_name_type_nodeid] = [preferred_terms_conceptid]
        tile.data[physical_thing_name_language_nodeid] = [english_conceptid]
        tile.save(transaction_id=transactionid, request=request, index=False)

        return tile

    def save_physical_thing_type(self, request, resourceid, transactionid, type):
        physical_thing_type_nodeid = "8ddfe3ab-b31d-11e9-aff0-a4d18cec433a"
        physical_thing_types = {
            "analysis_area": ["31d97bdd-f10f-4a26-958c-69cb5ab69af1"],
            "sample_area": ["7375a6fb-0bfb-4bcf-81a3-6180cdd26123"],
            "sample": ["77d8cf19-ce9c-4e0a-bde1-9148d870e11c"],
        }
        physical_thing_type = physical_thing_types[type]
        tile = self.save_node(
            request, resourceid, physical_thing_type_nodeid, physical_thing_type_nodeid, transactionid, physical_thing_type
        )
        return tile

    def save_physical_thing_related_collection(self, request, resourceinstanceid, transactionid, related_resourceid):
        physical_thing_member_of_nodeid = "63e49254-c444-11e9-afbe-a4d18cec433a"
        relationship_type = "31327077-8af5-4398-bbcc-e75675a9d37e"
        inverse_relationship_type = "6e7cf6a4-aba0-4a17-9a36-c69412212699"
        tile = self.save_related_resource_node(
            request,
            resourceinstanceid,
            physical_thing_member_of_nodeid,
            transactionid,
            related_resourceid,
            relationship_type=relationship_type,
            inverse_relationship_type=inverse_relationship_type,
        )
        return tile

    def save_physical_thing_part_of_tile(self, request, resourceid, transactionid, related_resourceid):
        physical_thing_part_of_nodeid = "f8d5fe4c-b31d-11e9-9625-a4d18cec433a"
        relationship_type = "da51c93e-950e-4801-90a7-31beff7d4f2b"
        inverse_relationship_type = "6d2969ea-41a2-4866-be92-14ba2b24f338"
        tile = self.save_related_resource_node(
            request,
            resourceid,
            physical_thing_part_of_nodeid,
            transactionid,
            related_resourceid,
            relationship_type=relationship_type,
            inverse_relationship_type=inverse_relationship_type,
        )
        return tile

    def save_parent_physical_thing_part_of_tile(self, request, resourceid, related_resourceid, transactionid, tiledata, tileid):
        part_identifier_assignment_nodegroupid = "fec59582-8593-11ea-97eb-acde48001122"
        physical_part_of_object_nodeid = "b240c366-8594-11ea-97eb-acde48001122"
        relationship_type = "6d2969ea-41a2-4866-be92-14ba2b24f338"
        inverse_relationship_type = "02404924-40a4-44aa-bc07-e7b70e5cc718"
        related_resource_template = get_related_resource_template(related_resourceid, relationship_type, inverse_relationship_type)
        tiledata[physical_part_of_object_nodeid] = [related_resource_template]
        tile = self.save_tile(request, resourceid, part_identifier_assignment_nodegroupid, transactionid, tiledata, tileid)
        return tile


class SaveAnalysisAreaView(SaveAnnotationView):
    """
    Updates the Parent physical thing (9519cb4f-b25b-11e9-8c7b-a4d18cec433a)
        the annotation (Part Identifier Assignment) (fec59582-8593-11ea-97eb-acde48001122)

    Creates Analysis Area physical thing (9519cb4f-b25b-11e9-8c7b-a4d18cec433a)
        the name (b9c1d8a6-b497-11e9-876b-a4d18cec433a)
        the type (8ddfe3ab-b31d-11e9-aff0-a4d18cec433a)
        the parent object (part of) (f8d5fe4c-b31d-11e9-9625-a4d18cec433a)
        the related collection (memeber of) (63e49254-c444-11e9-afbe-a4d18cec433a)
    """

    def post(self, request):
        parent_physical_thing_resourceid = request.POST.get("parentPhysicalThingResourceid")
        parent_physical_thing_name = request.POST.get("parentPhysicalThingName")
        collection_resourceid = request.POST.get("collectionResourceid")
        transaction_id = request.POST.get("transactionId")
        part_identifier_assignment_tile_data = JSONDeserializer().deserialize(request.POST.get("partIdentifierAssignmentTileData"))
        part_identifier_assignment_tile_id = request.POST.get("partIdentifierAssignmentTileId") or None
        part_identifier_assignment_label_nodeid = "3e541cc6-859b-11ea-97eb-acde48001122"
        physical_part_of_object_nodeid = "b240c366-8594-11ea-97eb-acde48001122"
        analysis_area_physical_thing_resourceid = None
        if part_identifier_assignment_tile_data[physical_part_of_object_nodeid]:
            analysis_area_physical_thing_resourceid = part_identifier_assignment_tile_data[physical_part_of_object_nodeid][0]["resourceId"]

        base_name = part_identifier_assignment_tile_data[part_identifier_assignment_label_nodeid][get_language()]["value"]
        name = _("{analysisName} [Analysis Area of {physicalThingName}]").format(
            analysisName=base_name, physicalThingName=parent_physical_thing_name
        )

        try:
            with transaction.atomic():
                if analysis_area_physical_thing_resourceid is None:
                    analysis_area_physical_thing_resourceid = self.create_physical_thing_resource(request, transaction_id)

                name_tile = self.save_physical_thing_name(request, analysis_area_physical_thing_resourceid, transaction_id, name)
                type_tile = self.save_physical_thing_type(request, analysis_area_physical_thing_resourceid, transaction_id, "analysis_area")
                member_of_tile = self.save_physical_thing_related_collection(
                    request, analysis_area_physical_thing_resourceid, transaction_id, collection_resourceid
                )
                part_of_tile = self.save_physical_thing_part_of_tile(
                    request, analysis_area_physical_thing_resourceid, transaction_id, parent_physical_thing_resourceid
                )
                physical_part_of_object_tile = self.save_parent_physical_thing_part_of_tile(
                    request,
                    parent_physical_thing_resourceid,
                    analysis_area_physical_thing_resourceid,
                    transaction_id,
                    part_identifier_assignment_tile_data,
                    part_identifier_assignment_tile_id,
                )

        except Exception as e:
            logger.exception(e)
            response = {"result": e, "message": [_("Request Failed"), _("Unable to save")]}
            return JSONResponse(response, status=500)

        res = {
            "nameTile": name_tile,
            "typeTile": type_tile,
            "memberOfTile": member_of_tile,
            "partOfTile": part_of_tile,
            "physicalPartOfObjectTile": physical_part_of_object_tile,
        }
        analysis_area_physical_thing_resource = Resource.objects.get(pk=analysis_area_physical_thing_resourceid)
        analysis_area_physical_thing_resource.index()
        parent_physical_thing_resource = Resource.objects.get(pk=parent_physical_thing_resourceid)
        parent_physical_thing_resource.index()
        return JSONResponse({"result": res})


class SaveSampleAreaView(SaveAnnotationView):
    def save_sampling_unit_tile(
        self,
        request,
        sampling_activity_resourceid,
        parent_physical_thing_resourceid,
        sample_area_physical_thing_resourceid,
        sample_physical_thing_resourceid,
        part_identifier_assignment_tile_data,
        transactionid,
        tileid=None,
    ):
        sampling_unit_nodegroupid = "b3e171a7-1d9d-11eb-a29f-024e0d439fdb"
        overall_object_sampled_nodeid = "b3e171aa-1d9d-11eb-a29f-024e0d439fdb"
        sampling_area_nodeid = "b3e171ac-1d9d-11eb-a29f-024e0d439fdb"
        sampling_area_sample_created_nodeid = "b3e171ab-1d9d-11eb-a29f-024e0d439fdb"
        sampling_area_visualization_nodeid = "b3e171ae-1d9d-11eb-a29f-024e0d439fdb"
        part_identifier_assignment_polygon_identifier_nodeid = "97c30c42-8594-11ea-97eb-acde48001122"
        sample_area_visualization = part_identifier_assignment_tile_data[part_identifier_assignment_polygon_identifier_nodeid]

        # relationship valueids
        overall_object_sampled = "4d8bb347-023d-498c-af87-4cb004ac938e"
        sampled_by = "fe4e083a-d764-4eee-806c-3b96beb2cdbd"
        sample_area_name = "1caa8744-6bc4-4f40-839e-1d96715cfa85"
        sample_area_identified_in = "dee862c0-954e-41e1-95e8-30102e39265f"
        sample_created = "0233939a-4b2b-4695-8020-9604c87bfdf4"
        sample_created_by = "ea975259-b349-43b9-85fb-8c762a41c484"

        if tileid is not None:
            tile = Tile.objects.get(pk=tileid)
        else:
            tile = None
            try:
                tiles = Tile.objects.filter(resourceinstance=sampling_activity_resourceid, nodegroup_id=sampling_unit_nodegroupid)
                for t in tiles:
                    if t.data[sampling_area_nodeid][0]["resourceId"] == sample_area_physical_thing_resourceid:
                        tile = t
                if tile is None:
                    tile = Tile.get_blank_tile_from_nodegroup_id(
                        nodegroup_id=sampling_unit_nodegroupid, resourceid=sampling_activity_resourceid
                    )

            except ObjectDoesNotExist as e:
                tile = Tile.get_blank_tile_from_nodegroup_id(
                    nodegroup_id=sampling_unit_nodegroupid, resourceid=sampling_activity_resourceid
                )

        tile.data[overall_object_sampled_nodeid] = [
            get_related_resource_template(parent_physical_thing_resourceid, overall_object_sampled, sampled_by)
        ]
        tile.data[sampling_area_nodeid] = [
            get_related_resource_template(sample_area_physical_thing_resourceid, sample_area_name, sample_area_identified_in)
        ]
        tile.data[sampling_area_sample_created_nodeid] = [
            get_related_resource_template(sample_physical_thing_resourceid, sample_created, sample_created_by)
        ]
        tile.data[sampling_area_visualization_nodeid] = sample_area_visualization

        tile.save(transaction_id=transactionid, request=request, index=False)

        return tile

    def save_sample_statement_tile(self, request, resourceid, statement, type, tileid=None):
        statement_nodegroupid = "1952bb0a-b498-11e9-a679-a4d18cec433a"
        statement_type_nodeid = "1952e470-b498-11e9-b261-a4d18cec433a"
        statement_content_nodeid = "1953016e-b498-11e9-9445-a4d18cec433a"
        statement_language_nodeid = "1952d7de-b498-11e9-a8a8-a4d18cec433a"
        english_conceptid = "bc35776b-996f-4fc1-bd25-9f6432c1f349"

        statement_types = {
            "motivation": "7060892c-4d91-4ab3-b3de-a95e19931a61",
            "description": "9886efe9-c323-49d5-8d32-5c2a214e5630",
        }

        if tileid is not None:
            tile = Tile.objects.get(pk=tileid)
        else:
            tile = None
            try:
                tiles = Tile.objects.filter(resourceinstance=resourceid, nodegroup_id=statement_nodegroupid)
                for t in tiles:
                    if statement_types[type] in t.data[statement_type_nodeid]:
                        tile = t
                if tile is None:
                    tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=statement_nodegroupid, resourceid=resourceid)

            except ObjectDoesNotExist as e:
                tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=statement_nodegroupid, resourceid=resourceid)

        tile.data[statement_content_nodeid] = {}
        tile.data[statement_content_nodeid][get_language()] = {"value": statement, "direction": "rtl" if get_language_bidi() else "ltr"}
        tile.data[statement_type_nodeid] = [statement_types[type]]
        tile.data[statement_language_nodeid] = [english_conceptid]
        tile.save(request=request, index=False)

        return tile

    def save_removed_from_tile(self, request, sample_resourceid, removed_from_resourceids, transactionid):
        removed_from_nodeid = "38814345-d2bd-11e9-b9d6-a4d18cec433a"
        removal_from_object_nodegroupid = "b11f217a-d2bc-11e9-8dfa-a4d18cec433a"

        # relationship valueids
        physical_object_removed_by_part_removal_event = "6fc21c46-2106-4802-b768-fe4f36f7c6e0"
        physical_object_removed_by_part_removal_event_of = "094c0d85-a6e1-41bd-992e-6fdf0fb2e1c4"

        removed_from_related_list = [
            get_related_resource_template(
                resourceid, physical_object_removed_by_part_removal_event, physical_object_removed_by_part_removal_event_of
            )
            for resourceid in removed_from_resourceids
        ]
        tile = self.save_node(
            request, sample_resourceid, removal_from_object_nodegroupid, removed_from_nodeid, transactionid, removed_from_related_list
        )
        tile.save(transaction_id=transactionid, request=request, index=False)

        return tile

    def post(self, request):
        physical_part_of_object_nodeid = "b240c366-8594-11ea-97eb-acde48001122"
        part_identifier_assignment_label_nodeid = "3e541cc6-859b-11ea-97eb-acde48001122"
        part_identifier_assignment_polygon_identifier_nodeid = "97c30c42-8594-11ea-97eb-acde48001122"
        sampling_area_nodeid = "b3e171ac-1d9d-11eb-a29f-024e0d439fdb"
        sampling_area_sample_created_nodeid = "b3e171ab-1d9d-11eb-a29f-024e0d439fdb"
        sampling_unit_nodegroupid = "b3e171a7-1d9d-11eb-a29f-024e0d439fdb"

        parent_physical_thing_resourceid = request.POST.get("parentPhysicalThingResourceid")
        parent_physical_thing_name = request.POST.get("parentPhysicalThingName")
        sampling_activity_resourceid = request.POST.get("samplingActivityResourceId")
        collection_resourceid = request.POST.get("collectionResourceid")
        sample_motivation = request.POST.get("sampleMotivation")
        sample_description = request.POST.get("sampleDescription")
        transaction_id = request.POST.get("transactionId")
        part_identifier_assignment_tile_data = JSONDeserializer().deserialize(request.POST.get("partIdentifierAssignmentTileData"))
        part_identifier_assignment_tile_id = request.POST.get("partIdentifierAssignmentTileId") or None

        sample_area_physical_thing_resourceid = None
        if part_identifier_assignment_tile_data[physical_part_of_object_nodeid]:
            sample_area_physical_thing_resourceid = part_identifier_assignment_tile_data[physical_part_of_object_nodeid][0]["resourceId"]

        sample_physical_thing_resourceid = None
        sampling_unit_tiles = Tile.objects.filter(resourceinstance=sampling_activity_resourceid, nodegroup_id=sampling_unit_nodegroupid)
        for sampling_unit_tile in sampling_unit_tiles:
            if sampling_unit_tile.data[sampling_area_nodeid][0]["resourceId"] == sample_area_physical_thing_resourceid:
                sample_physical_thing_resourceid = sampling_unit_tile.data[sampling_area_sample_created_nodeid][0]["resourceId"]

        base_name = part_identifier_assignment_tile_data[part_identifier_assignment_label_nodeid][get_language()]["value"]
        sample_name = _("{sampleName} [Sample of {physicalThingName}]").format(
            sampleName=base_name, physicalThingName=parent_physical_thing_name
        )
        sample_area_name = _("{sampleName} [Sample Area of {physicalThingName}]").format(
            sampleName=base_name, physicalThingName=parent_physical_thing_name
        )

        try:
            with transaction.atomic():
                # saving the sample area resource and tiles
                if sample_area_physical_thing_resourceid is None:
                    sample_area_physical_thing_resourceid = self.create_physical_thing_resource(request, transaction_id)

                sample_area_name_tile = self.save_physical_thing_name(
                    request, sample_area_physical_thing_resourceid, transaction_id, sample_area_name
                )
                sample_area_type_tile = self.save_physical_thing_type(
                    request, sample_area_physical_thing_resourceid, transaction_id, "sample_area"
                )
                sample_area_member_of_tile = self.save_physical_thing_related_collection(
                    request, sample_area_physical_thing_resourceid, transaction_id, collection_resourceid
                )
                sample_area_part_of_tile = self.save_physical_thing_part_of_tile(
                    request, sample_area_physical_thing_resourceid, transaction_id, parent_physical_thing_resourceid
                )

                # saving the sample resource and tiles
                if sample_physical_thing_resourceid is None:
                    sample_physical_thing_resourceid = self.create_physical_thing_resource(request, transaction_id)

                sample_name_tile = self.save_physical_thing_name(request, sample_physical_thing_resourceid, transaction_id, sample_name)
                sample_type_tile = self.save_physical_thing_type(request, sample_physical_thing_resourceid, transaction_id, "sample")
                sample_member_of_tile = self.save_physical_thing_related_collection(
                    request, sample_physical_thing_resourceid, transaction_id, collection_resourceid
                )
                sample_part_of_tile = self.save_physical_thing_part_of_tile(
                    request, sample_physical_thing_resourceid, transaction_id, parent_physical_thing_resourceid
                )
                removed_from_tile = self.save_removed_from_tile(
                    request,
                    sample_physical_thing_resourceid,
                    [parent_physical_thing_resourceid, sample_area_physical_thing_resourceid],
                    transaction_id,
                )

                # saving the sampling activity resource and tiles
                sampling_unit_tile = self.save_sampling_unit_tile(
                    request,
                    sampling_activity_resourceid,
                    parent_physical_thing_resourceid,
                    sample_area_physical_thing_resourceid,
                    sample_physical_thing_resourceid,
                    part_identifier_assignment_tile_data,
                    transaction_id,
                )

                sample_description_tile = self.save_sample_statement_tile(
                    request, sample_physical_thing_resourceid, sample_description, "description"
                )

                sample_motivation_tile = self.save_sample_statement_tile(
                    request, sample_physical_thing_resourceid, sample_motivation, "motivation"
                )

                # saving the parent physical thing area resource and tiles
                physical_part_of_object_tile = self.save_parent_physical_thing_part_of_tile(
                    request,
                    parent_physical_thing_resourceid,
                    sample_area_physical_thing_resourceid,
                    transaction_id,
                    part_identifier_assignment_tile_data,
                    part_identifier_assignment_tile_id,
                )
        except Exception as e:
            logger.exception(e)
            response = {"result": e, "message": [_("Request Failed"), _("Unable to save")]}
            return JSONResponse(response, status=500)

        resources_to_index = [
            sample_area_physical_thing_resourceid,
            sample_physical_thing_resourceid,
            sampling_activity_resourceid,
            parent_physical_thing_resourceid,
        ]
        for resource in Resource.objects.filter(pk__in=resources_to_index):
            resource.index()

        res = {
            "sample": {
                "nameTile": sample_name_tile,
                "typeTile": sample_type_tile,
                "memberOfTile": sample_member_of_tile,
                "partOfTile": sample_part_of_tile,
                "removedFromTile": removed_from_tile,
            },
            "sampleArea": {
                "nameTile": sample_area_name_tile,
                "typeTile": sample_area_type_tile,
                "memberOfTile": sample_area_member_of_tile,
                "partOfTile": sample_area_part_of_tile,
            },
            "samplingActivity": {
                "samplingUnitTile": sampling_unit_tile,
                "samplingDescriptionTile": sample_description_tile,
                "samplingMotivationTile": sample_motivation_tile,
            },
            "parentPhysicalThing": {
                "physicalPartOfObjectTile": physical_part_of_object_tile,
            },
        }

        return JSONResponse({"result": res})


@method_decorator(csrf_exempt, name="dispatch")
class DeleteSampleAreaView(View):
    def post(self, request):
        # need to delete:
        # the "sample" physical thing
        # the "sample area" physical thing
        # the tile from the sampling activity that references the "sample area"

        data = JSONDeserializer().deserialize(request.body)

        physical_part_of_object_nodeid = "b240c366-8594-11ea-97eb-acde48001122"
        part_identifier_assignment_label_nodeid = "3e541cc6-859b-11ea-97eb-acde48001122"
        part_identifier_assignment_polygon_identifier_nodeid = "97c30c42-8594-11ea-97eb-acde48001122"
        sampling_area_nodeid = "b3e171ac-1d9d-11eb-a29f-024e0d439fdb"
        sampling_area_sample_created_nodeid = "b3e171ab-1d9d-11eb-a29f-024e0d439fdb"
        sampling_unit_nodegroupid = "b3e171a7-1d9d-11eb-a29f-024e0d439fdb"

        sampling_activity_resourceid = data.get("samplingActivityResourceId")
        collection_resourceid = data.get("collectionResourceid")
        sample_motivation = data.get("sampleMotivation")
        sample_description = data.get("sampleDescription")
        transaction_id = data.get("transactionId")

        part_identifier_assignment_tile_data = JSONDeserializer().deserialize(data.get("partIdentifierAssignmentTileData"))
        part_identifier_assignment_tile_id = data.get("partIdentifierAssignmentTileId", None)

        sample_area_physical_thing_resourceid = None
        if part_identifier_assignment_tile_data[physical_part_of_object_nodeid]:
            sample_area_physical_thing_resourceid = part_identifier_assignment_tile_data[physical_part_of_object_nodeid][0]["resourceId"]

        try:
            sample_physical_thing_resourceid = None
            sampling_unit_tiles = Tile.objects.filter(
                resourceinstance_id=sampling_activity_resourceid, nodegroup_id=sampling_unit_nodegroupid
            )
            for sampling_unit_tile in sampling_unit_tiles:
                if sampling_unit_tile.data[sampling_area_nodeid][0]["resourceId"] == sample_area_physical_thing_resourceid:
                    sample_physical_thing_resourceid = sampling_unit_tile.data[sampling_area_sample_created_nodeid][0]["resourceId"]

            samplingActivitySampleTile = ResourceXResource.objects.get(
                nodeid=sampling_area_nodeid,
                resourceinstanceidfrom_id=sampling_activity_resourceid,
                resourceinstanceidto_id=sample_area_physical_thing_resourceid,
            )

            with transaction.atomic():
                Resource.objects.get(resourceinstanceid=sample_area_physical_thing_resourceid).delete(
                    transaction_id=transaction_id, user=request.user
                )
                Resource.objects.get(resourceinstanceid=sample_physical_thing_resourceid).delete(
                    transaction_id=transaction_id, user=request.user
                )

                Tile.objects.get(tileid=samplingActivitySampleTile.tileid_id).delete(transaction_id=transaction_id, request=request)
            return JSONResponse(status=200)
        except:
            response = {"message": _("Unable to delete"), "title": _("Delete Failed")}
            return JSONResponse(response, status=500)


@method_decorator(csrf_exempt, name="dispatch")
class DeleteAnalysisAreaView(View):
    def post(self, request):
        # need to delete:
        # the "analysis" physical thing

        data = JSONDeserializer().deserialize(request.body)
        part_identifier_assignment_tile_data = JSONDeserializer().deserialize(data.get("parentPhysicalThingTileData"))
        transaction_id = data.get("transactionId")

        physical_part_of_object_nodeid = "b240c366-8594-11ea-97eb-acde48001122"
        analysis_area_physical_thing_resourceid = None

        if part_identifier_assignment_tile_data[physical_part_of_object_nodeid]:
            analysis_area_physical_thing_resourceid = part_identifier_assignment_tile_data[physical_part_of_object_nodeid][0]["resourceId"]

        try:
            with transaction.atomic():
                Resource.objects.get(resourceinstanceid=analysis_area_physical_thing_resourceid).delete(
                    transaction_id=transaction_id, user=request.user
                )
            return JSONResponse(status=200)
        except:
            response = {"message": _("Unable to delete"), "title": _("Delete Failed")}
            return JSONResponse(response, status=500)


class GetLockedStatus(View):
    def get(self, request):
        digitalResourceGraphId = "707cbd78-ca7a-11e9-990b-a4d18cec433a"
        resourceId = request.GET.get("resourceId")
        ret = ResourceXResource.objects.filter(
            nodeid="a298ee52-8d59-11eb-a9c4-faffc265b501",
            resourceinstanceidfrom_id=resourceId,
            resourceinstanceto_graphid_id=digitalResourceGraphId,
        ).exists()

        return JSONResponse({"isRelatedToDigitalResource": ret})



# Generated by Django 4.2.4 on 2023-10-17 19:09
from django.db import migrations
from guardian.ctypes import get_content_type


def assign_view_plugin(apps, schema_editor):
    Group = apps.get_model("auth", "Group")
    Plugin = apps.get_model("models", "Plugin")
    Permission = apps.get_model("auth", "Permission")
    GroupObjectPermission = apps.get_model("guardian", "GroupObjectPermission")

    view_plugin = Permission.objects.get(codename="view_plugin")
    resource_reviewers = Group.objects.get(name="Resource Reviewer")
    plugins = Plugin.objects.all()

    # Cannot use django_guardian shortcuts or object managers
    # https://github.com/django-guardian/django-guardian/issues/751
    GroupObjectPermission.objects.bulk_create(
        [
            GroupObjectPermission(
                permission=view_plugin,
                group=resource_reviewers,
                content_type_id=get_content_type(plugin).pk,
                object_pk=plugin.pk,
            )
            for plugin in plugins
        ],
        ignore_conflicts=True,
    )


class Migration(migrations.Migration):
    dependencies = [
        ("arches_for_science", "0001_initial"),
        ("guardian", "0002_generic_permissions_index"),
    ]

    operations = [
        migrations.RunPython(assign_view_plugin, migrations.RunPython.noop),
    ]



import uuid
from arches.app.models.models import TileModel, FunctionXGraph
from django.db import models
from django.db.models import JSONField
import pgtrigger

from .trigger_functions import CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_SINGLE, CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_ALL


class TileModelProxy(TileModel):
    class Meta:
        proxy = True
        triggers = [
            pgtrigger.Trigger(
                name="calculate_multicard_primary_descriptor_single",
                when=pgtrigger.After,
                operation=pgtrigger.Insert | pgtrigger.Update | pgtrigger.Delete,
                func=CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_SINGLE,
            ),
        ]


class FunctionXGraphProxy(FunctionXGraph):
    class Meta:
        proxy = True
        triggers = [
            pgtrigger.Trigger(
                name="calculate_multicard_primary_descriptor_all",
                when=pgtrigger.After,
                condition=pgtrigger.Q(
                    new__function_id="00b2d15a-fda0-4578-b79a-784e4138664b",
                    new__config__isnull=False,
                ),
                operation=pgtrigger.Insert | pgtrigger.Update,
                func=CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_ALL,
            ),
        ]


class RendererConfig(models.Model):
    configid = models.UUIDField(primary_key=True, unique=True)
    rendererid = models.UUIDField()
    name = models.TextField(blank=False, null=False)
    description = models.TextField(blank=True, null=True)
    config = JSONField(default=dict)

    class Meta:
        managed = True
        db_table = "renderer_config"

    def __init__(self, *args, **kwargs):
        super(RendererConfig, self).__init__(*args, **kwargs)
        if not self.configid:
            self.configid = uuid.uuid4()


class ManifestXDigitalResource(models.Model):
    manifest = models.TextField(unique=True)
    digitalresource = models.TextField(unique=True)

    class Meta:
        managed = True
        db_table = "manifest_x_digitalresource"


class CanvasXDigitalResource(models.Model):
    canvas = models.TextField(unique=True)
    digitalresource = models.TextField(unique=True)

    class Meta:
        managed = True
        db_table = "canvas_x_digitalresource"


class ManifestXCanvas(models.Model):
    manifest = models.TextField()
    canvas = models.TextField()

    class Meta:
        managed = True
        db_table = "manifest_x_canvas"



# Generated by Django 4.2.4 on 2023-11-20 13:43

from django.db import migrations
from django.utils.translation import gettext_lazy as _


STRING_TEMPLATE_REPLACEMENTS = {
    "Statement_content": "statement_content",
    "Name_content": "name_content",
    "Name (top)_content": "name_top__content",
    "<content >": "<statement_content>",
}
STRING_TEMPLATE_REPLACEMENTS_REVERSED = {v: k for k, v in STRING_TEMPLATE_REPLACEMENTS.items()}

ORIGINAL_DESCRIPTOR_FUNCTION_PK = "60000000-0000-0000-0000-000000000001"
MULTICARD_DESCRIPTOR_FUNCTION_PK = "00b2d15a-fda0-4578-b79a-784e4138664b"


def update_graphs(apps, from_function, to_function, string_replacement_map):
    FunctionXGraph = apps.get_model("models", "FunctionXGraph")

    for fn_x_graph in FunctionXGraph.objects.filter(function=from_function):
        fn_x_graph.function = to_function
        for descriptor_object in fn_x_graph.config["descriptor_types"].values():
            for before, after in string_replacement_map.items():
                descriptor_object["string_template"] = descriptor_object["string_template"].replace(before, after)
        fn_x_graph.save()


def retire_core_primary_descriptor(apps, schema_editor):
    Function = apps.get_model("models", "Function")
    original_fn = Function.objects.get(pk=ORIGINAL_DESCRIPTOR_FUNCTION_PK)
    original_fn.functiontype = "primarydescriptors_retired"
    original_fn.name = "(retired) Define Resource Descriptors"
    original_fn.save()

    # create multi-card function
    multi_card_fn, created = Function.objects.update_or_create(
        pk=MULTICARD_DESCRIPTOR_FUNCTION_PK,
        modulename="multicard_resource_descriptor.py",
        classname="MulticardResourceDescriptor",
        functiontype="primarydescriptors",
        name=_("Multi-card Resource Descriptor"),
        description=_("Configure the name, description, and map popup of a resource"),
        defaultconfig=original_fn.defaultconfig,
        component="views/components/functions/multicard-resource-descriptor",
    )

    update_graphs(apps, from_function=original_fn, to_function=multi_card_fn, string_replacement_map=STRING_TEMPLATE_REPLACEMENTS)


def restore_core_primary_descriptor(apps, schema_editor):
    Function = apps.get_model("models", "Function")
    original_fn = Function.objects.get(pk=ORIGINAL_DESCRIPTOR_FUNCTION_PK)
    original_fn.functiontype = "primarydescriptors"
    original_fn.name = _("Define Resource Descriptors")
    original_fn.save()

    multi_card_fn = Function.objects.get(pk=MULTICARD_DESCRIPTOR_FUNCTION_PK)

    # Update FunctionXGraph records before cascade deleted below
    update_graphs(apps, from_function=multi_card_fn, to_function=original_fn, string_replacement_map=STRING_TEMPLATE_REPLACEMENTS_REVERSED)

    multi_card_fn.delete()


class Migration(migrations.Migration):
    dependencies = [
        ("arches_for_science", "0003_manifest_x_canvas_x_digitalresource"),
    ]

    operations = [
        migrations.RunPython(
            retire_core_primary_descriptor,
            restore_core_primary_descriptor,
        )
    ]



from http import HTTPStatus

from django.core.exceptions import ObjectDoesNotExist
from django.views.generic import View

from arches.app.utils.response import JSONErrorResponse, JSONResponse
from arches_for_science.models import ManifestXDigitalResource, CanvasXDigitalResource


class ManifestXCanvasView(View):
    def get(self, request):
        resourceid = request.GET.get("resourceid", None)
        manifest = request.GET.get("manifest", None)
        canvas = request.GET.get("canvas", None)
        if resourceid:
            try:
                link = ManifestXDigitalResource.objects.get(digitalresource=resourceid)
                manifest = link.manifest
            except ObjectDoesNotExist:
                link = CanvasXDigitalResource.objects.get(digitalresource=resourceid)
                canvas = link.canvas
            digital_resource = link.digitalresource
        elif manifest:
            digital_resource = ManifestXDigitalResource.objects.get(manifest=manifest).digitalresource
        elif canvas:
            digital_resource = CanvasXDigitalResource.objects.get(canvas=canvas).digitalresource
        else:
            return JSONErrorResponse(status=HTTPStatus.BAD_REQUEST)

        result = {"manifest": manifest, "canvas": canvas, "digital_resource": digital_resource}

        return JSONResponse(result)



# Generated by Django 4.2.4 on 2023-11-17 12:48

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("arches_for_science", "0002_view_plugin_permission"),
    ]

    operations = [
        migrations.CreateModel(
            name="CanvasXDigitalResource",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("canvas", models.TextField(unique=True)),
                ("digitalresource", models.TextField(unique=True)),
            ],
            options={
                "db_table": "canvas_x_digitalresource",
                "managed": True,
            },
        ),
        migrations.CreateModel(
            name="ManifestXCanvas",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("manifest", models.TextField()),
                ("canvas", models.TextField()),
            ],
            options={
                "db_table": "manifest_x_canvas",
                "managed": True,
            },
        ),
        migrations.CreateModel(
            name="ManifestXDigitalResource",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("manifest", models.TextField(unique=True)),
                ("digitalresource", models.TextField(unique=True)),
            ],
            options={
                "db_table": "manifest_x_digitalresource",
                "managed": True,
            },
        ),
    ]



from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("arches_for_science", "0005_deploy_triggers_for_primary_descriptor"),
    ]

    remove_top_concept = """
        update relations set conceptidfrom = 'b73e741b-46da-496c-8960-55cc1007bec4', relationtype = 'hasTopConcept' where conceptidfrom = '102efe26-d6b5-4524-a1bc-93b1792b6970';
        delete from values where conceptid = '102efe26-d6b5-4524-a1bc-93b1792b6970';
        delete from relations where conceptidto = '102efe26-d6b5-4524-a1bc-93b1792b6970';
        delete from concepts where conceptid = '102efe26-d6b5-4524-a1bc-93b1792b6970';
        """

    restore_top_concept = """
        insert into concepts (conceptid, legacyoid, nodetype) values ('102efe26-d6b5-4524-a1bc-93b1792b6970', 'http://localhost:8000/102efe26-d6b5-4524-a1bc-93b1792b6970', 'Concept');
        insert into values (valueid, value, conceptid, languageid, valuetype) values ('43a86d96-4b08-48f0-ac23-ccad248fdaec','TopConcept', '102efe26-d6b5-4524-a1bc-93b1792b6970', 'en-US', 'prefLabel');
        insert into values (valueid, value, conceptid, languageid, valuetype) values ('542946b6-b609-4145-afa0-449ec3a6ec0a','http://localhost:8000/102efe26-d6b5-4524-a1bc-93b1792b6970', '102efe26-d6b5-4524-a1bc-93b1792b6970', 'en', 'identifier');
        update relations set conceptidfrom = '102efe26-d6b5-4524-a1bc-93b1792b6970', relationtype = 'narrower' where conceptidfrom = 'b73e741b-46da-496c-8960-55cc1007bec4';
        insert into relations (relationid, conceptidfrom, conceptidto, relationtype) values ('cad00dfe-991e-4d5a-80b1-9dc359fdf528','b73e741b-46da-496c-8960-55cc1007bec4','102efe26-d6b5-4524-a1bc-93b1792b6970', 'hasTopConcept');
    """

    operations = [
        migrations.RunSQL(
            remove_top_concept,
            restore_top_concept,
        )
    ]



# Generated by Django 3.2.19 on 2023-07-13 22:08

from django.db import migrations, models
from django.utils.translation import gettext as _


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("models", "10121_workflowhistory"),
    ]

    def add_plugins(apps, schema_editor):
        Plugin = apps.get_model("models", "Plugin")

        # Add Chemical Analysis Images Workflow
        Plugin.objects.update_or_create(
            pluginid="af06e949-5e16-49f0-b23e-e8529e8ce321",
            name=_("Add Chemical Analysis Images"),
            icon="fa fa-pie-chart",
            component="views/components/plugins/add-chemical-analysis-images-workflow",
            componentname="add-chemical-analysis-images-workflow",
            config={
                "show": False,
                "is_workflow": True,
                "thumbnailBackgroundColor": "#7a7ecf",
                "thumbnailCircleColor": "#59699e",
                "description": "Add images to a chemical analysis",
            },
            slug="add-chemical-analysis-images-workflow",
            sortorder=1,
            helptemplate="add-chemical-analysis-images-workflow-help",
        )

        # Analysis Areas Workflow
        Plugin.objects.update_or_create(
            pluginid="29512c4e-ed06-4a6b-ae2d-e423817b3f6e",
            name=_("Analysis Areas"),
            icon="fa fa-pie-chart",
            component="views/components/plugins/analysis-areas-workflow",
            componentname="analysis-areas-workflow",
            config={
                "show": False,
                "is_workflow": True,
                "thumbnailBackgroundColor": "#6ea2d8",
                "thumbnailCircleColor": "#87b9e5",
                "description": "Define regions of interest on an object or sample",
            },
            slug="analysis-areas-workflow",
            sortorder=1,
            helptemplate="analysis-areas-workflow-help",
        )

        # Chemical Analysis Workflow
        Plugin.objects.update_or_create(
            pluginid="c206cfc6-6b4a-481e-a018-8da72aeb7074",
            name=_("Chemical Analysis Data Upload"),
            icon="fa fa-flask",
            component="views/components/plugins/chemical-analysis-workflow",
            componentname="chemical-analysis-workflow",
            config={
                "show": False,
                "is_workflow": True,
                "thumbnailBackgroundColor": "#5c61b7",
                "thumbnailCircleColor": "#455486",
                "description": "Upload chemical image files and select an image service",
            },
            slug="chemical-analysis-workflow",
            sortorder=1,
            helptemplate="chemical-analysis-workflow-help",
        )

        # Create Project Workflow
        Plugin.objects.update_or_create(
            pluginid="a1667717-b7bd-4570-b27a-ec352c767e0e",
            name=_("New Project"),
            icon="fa fa-file",
            component="views/components/plugins/create-project-workflow",
            componentname="create-project-workflow",
            config={
                "show": False,
                "is_workflow": True,
                "thumbnailBackgroundColor": "#9591ef",
                "thumbnailCircleColor": "#b0adf3",
                "description": "A research effort, study, or conservation project",
            },
            slug="create-project-workflow",
            sortorder=0,
            helptemplate="create-project-workflow-help",
        )

        # Init Workflow
        Plugin.objects.update_or_create(
            pluginid="49507fb0-89c6-47b7-b506-9b2b29a3b8d8",
            name=_("Workflows"),
            icon="fa fa-play-circle",
            component="views/components/plugins/init-workflow",
            componentname="init-workflow",
            config={"show": True},
            slug="init-workflow",
            sortorder=0,
        )

        # Project Collection Workflow
        Plugin.objects.update_or_create(
            pluginid="ef3a374c-f795-4317-aaee-83e1493eabb6",
            name=_("Update Project Collection"),
            icon="fa fa-file",
            component="views/components/plugins/project-collection-workflow",
            componentname="project-collection-workflow",
            config={
                "show": False,
                "is_workflow": True,
                "thumbnailBackgroundColor": "#3e89cd",
                "thumbnailCircleColor": "#6ea7d9",
                "description": "Add objects to an existing project",
            },
            slug="project-collection-workflow",
            sortorder=0,
            helptemplate="project-collection-workflow-help",
        )

        # Project Report Workflow
        Plugin.objects.update_or_create(
            pluginid="730be1cd-0575-44d7-be68-1316c345a8fb",
            name=_("Project Report"),
            icon="fa fa-paperclip",
            component="views/components/plugins/project-report-workflow",
            componentname="project-report-workflow",
            config={
                "show": False,
                "is_workflow": True,
                "thumbnailBackgroundColor": "#3e89cd",
                "thumbnailCircleColor": "#6ea7d9",
                "description": "Project Report Workflow",
            },
            slug="project-report-workflow",
            sortorder=1,
            helptemplate="project-report-workflow-help",
        )

        # Review Dataset Workflow
        Plugin.objects.update_or_create(
            pluginid="129d18d4-261a-4a3c-b46e-cf74e71d848c",
            name=_("Review Dataset"),
            icon="fa fa-files-o",
            component="views/components/plugins/review-dataset-workflow",
            componentname="review-dataset-workflow",
            config={
                "show": False,
                "is_workflow": True,
                "thumbnailBackgroundColor": "#3e89cd",
                "thumbnailCircleColor": "#6ea7d9",
                "description": "Review digital data files generated by a lab instrument",
            },
            slug="review-dataset-workflow",
            sortorder=1,
            helptemplate="review-dataset-workflow-help",
        )

        # Sample Taking Workflow
        Plugin.objects.update_or_create(
            pluginid="1132c953-8af4-4acb-87c5-39c5b996799e",
            name=_("Sample Taking"),
            icon="fa fa-file",
            component="views/components/plugins/sample-taking-workflow",
            componentname="sample-taking-workflow",
            config={
                "show": False,
                "is_workflow": True,
                "thumbnailBackgroundColor": "#87bdeb",
                "thumbnailCircleColor": "#a6cdf1",
                "description": "An activity to obtain a portion of an object for analysis",
            },
            slug="sample-taking-workflow",
            sortorder=1,
            helptemplate="sample-taking-workflow-help",
        )

        # Upload Dataset Workflow
        Plugin.objects.update_or_create(
            pluginid="afedd8cd-1ba7-43ec-b3c5-d0fab03525ca",
            name=_("Upload XY Data"),
            icon="fa fa-upload",
            component="views/components/plugins/upload-dataset-workflow",
            componentname="upload-dataset-workflow",
            config={
                "show": False,
                "is_workflow": True,
                "thumbnailBackgroundColor": "#408acd",
                "thumbnailCircleColor": "#6fa8d9",
                "description": "Digital spectral data files generated by a lab instrument",
            },
            slug="upload-dataset-workflow",
            sortorder=1,
            helptemplate="upload-dataset-workflow-help",
        )

    def remove_plugins(apps, schema_editor):
        Plugin = apps.get_model("models", "Plugin")

        for plugin in Plugin.objects.filter(
            pk__in=[
                "af06e949-5e16-49f0-b23e-e8529e8ce321",  # Add Chemical Analysis Images Workflow
                "29512c4e-ed06-4a6b-ae2d-e423817b3f6e",  # Analysis Areas Workflow
                "c206cfc6-6b4a-481e-a018-8da72aeb7074",  # Chemical Analysis Workflow
                "a1667717-b7bd-4570-b27a-ec352c767e0e",  # Create Project Workflow
                "49507fb0-89c6-47b7-b506-9b2b29a3b8d8",  # Init Workflow
                "ef3a374c-f795-4317-aaee-83e1493eabb6",  # Project Collection Workflow
                "730be1cd-0575-44d7-be68-1316c345a8fb",  # Project Report Workflow
                "1132c953-8af4-4acb-87c5-39c5b996799e",  # Sample Taking Workflow
                "afedd8cd-1ba7-43ec-b3c5-d0fab03525ca",  # Upload Dataset Workflow
            ]
        ):
            plugin.delete()

        # Deletes current review-dataset workflow if it exists, it has an invalid pluginid
        review_dataset_workflow_plugin_query = Plugin.objects.filter(componentname="review-dataset-workflow")
        if len(review_dataset_workflow_plugin_query):
            review_dataset_workflow_plugin = review_dataset_workflow_plugin_query[0]
            review_dataset_workflow_plugin.delete()

    def add_reports(apps, schema_editor):
        ReportTemplate = apps.get_model("models", "ReportTemplate")

        # Collection or Set Template
        ReportTemplate.objects.update_or_create(
            name=_("Collection or Set Template"),
            component="views/components/reports/collection-set",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Collection or Set report"),
            componentname="collection-set-report",
            templateid="b5380d34-15ef-4a38-95c0-df228068916d",
            preload_resource_data=False,
        )

        # Digital Resource Template
        ReportTemplate.objects.update_or_create(
            name=_("Digital Resource Template"),
            component="views/components/reports/digital-resource",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Digital Resource report"),
            componentname="digital-resource-report",
            templateid="e6390e54-31c4-4055-b064-ea9296c1c13a",
            preload_resource_data=False,
        )

        # Group Template
        ReportTemplate.objects.update_or_create(
            name=_("Group Template"),
            component="views/components/reports/group",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Group report"),
            componentname="group-report",
            templateid="3a9fd7d1-f065-4777-b664-3fda0eb39ffe",
            preload_resource_data=False,
        )

        # Instrument Template
        ReportTemplate.objects.update_or_create(
            name=_("Instrument Template"),
            component="views/components/reports/instrument",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Instrument report"),
            componentname="instrument-report",
            templateid="c2edf720-9db0-4029-a102-48e6762ded79",
            preload_resource_data=False,
        )

        # Modification Template
        ReportTemplate.objects.update_or_create(
            name=_("Modification Template"),
            component="views/components/reports/modification",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Modification report"),
            componentname="modification-report",
            templateid="f02f4b69-08d9-49fa-a4d1-94ef8178fbce",
            preload_resource_data=False,
        )

        # Observation Template
        ReportTemplate.objects.update_or_create(
            name=_("Observation Template"),
            component="views/components/reports/observation",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Observation report"),
            componentname="observation-report",
            templateid="e581f911-185b-4456-873d-f83b46fd89ec",
            preload_resource_data=False,
        )

        # Period Template
        ReportTemplate.objects.update_or_create(
            name=_("Period Template"),
            component="views/components/reports/period",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Period report"),
            componentname="period-report",
            templateid="9f6b087b-12f4-4436-8bfc-257d1665ba40",
            preload_resource_data=False,
        )

        # Person Template
        ReportTemplate.objects.update_or_create(
            name=_("Person Template"),
            component="views/components/reports/person",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Person report"),
            componentname="person-report",
            templateid="15d3ab75-1191-41f0-a766-e7e8ce7b9ca9",
            preload_resource_data=False,
        )

        # Physical Thing Template
        ReportTemplate.objects.update_or_create(
            name=_("Physical Thing Template"),
            component="views/components/reports/physical-thing",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Physical Thing report"),
            componentname="physical-thing-report",
            templateid="b3056c21-33c4-4055-b064-ea9296c1c13a",
            preload_resource_data=False,
        )

        # Place Template
        ReportTemplate.objects.update_or_create(
            name=_("Place Template"),
            component="views/components/reports/place",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Place report"),
            componentname="place-report",
            templateid="34b7109d-2c0a-47fa-a307-4e361e142bfe",
            preload_resource_data=False,
        )

        # Project Template
        ReportTemplate.objects.update_or_create(
            name=_("Project Template"),
            component="views/components/reports/project",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Project report"),
            componentname="project-report",
            templateid="2b124304-6e42-4c0c-8161-14c010dcef71",
            preload_resource_data=False,
        )

        # Provenance Activity Template
        ReportTemplate.objects.update_or_create(
            name=_("Provenance Activity Template"),
            component="views/components/reports/provenance-activity",
            defaultconfig={"uncompacted_reporting": True},
            description=_("provenance Activity report"),
            componentname="provenance-activity-report",
            templateid="1a51566c-4ba0-4633-b717-18f629350c9c",
            preload_resource_data=False,
        )

        # Sampling Activity Template
        ReportTemplate.objects.update_or_create(
            name=_("Sampling Activity Template"),
            component="views/components/reports/sampling-activity",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Sampling Activity report"),
            componentname="sampling-activity-report",
            templateid="4c864659-fc0d-45ec-a4da-c8fb8a7ec24e",
            preload_resource_data=False,
        )

        # Textual Work Template
        ReportTemplate.objects.update_or_create(
            name=_("Textual Work Template"),
            component="views/components/reports/textual-work",
            defaultconfig={"uncompacted_reporting": True},
            description=_("Textual Work report"),
            componentname="textual-work-report",
            templateid="590a02ad-7d47-4b29-90a3-2e29aada677d",
            preload_resource_data=False,
        )

        # Thematic Work Template
        ReportTemplate.objects.update_or_create(
            name=_("Thematic Template"),
            component="views/components/reports/thematic",
            defaultconfig={},
            description=_("Thematic report template"),
            componentname="thematic-report",
            preload_resource_data=False,
            templateid="c560f22a-99b9-4a1c-b805-0891bbe63ada",
        )

        # Visual Work Template
        ReportTemplate.objects.update_or_create(
            name=_("Visual Work Template"),
            component="views/components/reports/visual-work",
            defaultconfig={
                "tabs": [
                    {"icon": "fa fa-info", "name": "Identity", "nodegroup_ids": []},
                    {"icon": "fa fa-chain", "name": "Links", "nodegroup_ids": []},
                ],
                "activeTabIndex": 1,
            },
            description=_("Visual Work report"),
            componentname="visual-work-report",
            templateid="c4167d32-33c4-4055-b064-ea9296c1c13a",
        )

    def remove_reports(apps, schema_editor):
        ReportTemplate = apps.get_model("models", "ReportTemplate")

        for report_template in ReportTemplate.objects.filter(
            pk__in=[
                "b5380d34-15ef-4a38-95c0-df228068916d",  # Collection or Set Template
                "e6390e54-31c4-4055-b064-ea9296c1c13a",  # Digital Resource Template
                "3a9fd7d1-f065-4777-b664-3fda0eb39ffe",  # Group Template
                "c2edf720-9db0-4029-a102-48e6762ded79",  # Instrument Template
                "f02f4b69-08d9-49fa-a4d1-94ef8178fbce",  # Modification Template
                "e581f911-185b-4456-873d-f83b46fd89ec",  # Observation Template
                "9f6b087b-12f4-4436-8bfc-257d1665ba40",  # Period Template
                "15d3ab75-1191-41f0-a766-e7e8ce7b9ca9",  # Person Template
                "b3056c21-33c4-4055-b064-ea9296c1c13a",  # Physical Thing Template
                "34b7109d-2c0a-47fa-a307-4e361e142bfe",  # Place Template
                "2b124304-6e42-4c0c-8161-14c010dcef71",  # Project Template
                "1a51566c-4ba0-4633-b717-18f629350c9c",  # Provenance Activity Template
                "4c864659-fc0d-45ec-a4da-c8fb8a7ec24e",  # Sampling Activity Template
                "590a02ad-7d47-4b29-90a3-2e29aada677d",  # Textual Work Template
                "c560f22a-99b9-4a1c-b805-0891bbe63ada",  # Thematic Work Template
                "c4167d32-33c4-4055-b064-ea9296c1c13a",  # Visual Work Template
            ]
        ):
            report_template.delete()

    def add_card_components(apps, schema_editor):
        CardComponent = apps.get_model("models", "CardComponent")

        # Physical Thing IIIF Card
        CardComponent.objects.update_or_create(
            componentid="10ae2f00-cbda-4a0d-8ca2-5af3b46b37ad",
            name=_("Physical Thing IIIF Card"),
            description="",
            component="views/components/cards/physical-thing-iiif-card",
            componentname="physical-thing-iiif-card",
            defaultconfig={"defaultManifest": ""},
        )

    def remove_card_components(apps, schema_editor):
        CardComponent = apps.get_model("models", "CardComponent")

        for card_component in CardComponent.objects.filter(
            pk__in=[
                "10ae2f00-cbda-4a0d-8ca2-5af3b46b37ad",  # Physical Thing IIIF Card
            ]
        ):
            card_component.delete()

    operations = [
        migrations.CreateModel(
            name="RendererConfig",
            fields=[
                ("configid", models.UUIDField(primary_key=True, serialize=False, unique=True)),
                ("rendererid", models.UUIDField()),
                ("name", models.TextField()),
                ("description", models.TextField(blank=True, null=True)),
                ("config", models.JSONField(default=dict)),
            ],
            options={
                "db_table": "renderer_config",
                "managed": True,
            },
        ),
        migrations.RunPython(add_plugins, remove_plugins),
        migrations.RunPython(add_reports, remove_reports),
        migrations.RunPython(add_card_components, remove_card_components),
    ]



#!/usr/bin/env python

"""
ARCHES - a program developed to inventory and manage immovable cultural heritage.
Copyright (C) 2013 J. Paul Getty Trust and World Monuments Fund

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""


import os
import sys

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "arches_for_science.settings")

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



from celery import shared_task
from django.contrib.auth.models import User
from django.http import HttpRequest
from django.utils.translation import gettext as _
from arches.app.models.models import SearchExportHistory
from arches.app.models.system_settings import settings


try:
    from arches.app.tasks import *
except ImportError:
    pass


@shared_task
def download_project_files_task(userid, files, temp_files, project_name):
    from arches_for_science.views.download_project_files import FileDownloader

    downloader = FileDownloader()
    user = User.objects.get(pk=userid)
    exportid, files = downloader.create_download_zipfile(user, files, temp_files, project_name)

    search_history_obj = SearchExportHistory.objects.get(pk=exportid) if exportid else None

    # TODO(i18n): should this be a gettext_lazy?
    msg = _("The report(s) and the related file(s) for the project '{}' are ready for download.").format(project_name)
    notiftype_name = "Search Export Download Ready"
    context = dict(
        greeting=_("Your request to download related project files is complete and your files are ready for download."),
        link=str(exportid) if exportid else "",
        files=files,
        button_text=_("Download Now"),
        closing=_("Thank you"),
        email=user.email,
        email_link=(
            str(settings.ARCHES_NAMESPACE_FOR_DATA_EXPORT).rstrip("/") + settings.MEDIA_URL + str(search_history_obj.downloadfile)
            if search_history_obj
            else None
        ),
    )

    notify_completion(msg, user, notiftype_name, context)



from django.core.management import call_command
from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse
from django.utils.translation import get_language

from arches.app.utils.betterJSONSerializer import JSONSerializer
from arches.app.models.models import GraphModel, Node, NodeGroup, ResourceInstance, ResourceXResource, TileModel
from arches.app.models.tile import Tile

PHYSICAL_THING_GRAPH_ID = "9519cb4f-b25b-11e9-8c7b-a4d18cec433a"
PART_IDENTIFIER_ASSIGNMENT = "b240c366-8594-11ea-97eb-acde48001122"
PART_IDENTIFIER_ASSIGNMENT_LABEL = "3e541cc6-859b-11ea-97eb-acde48001122"
SAMPLING_ACTIVITY_GRAPH_ID = "03357848-1d9d-11eb-a29f-024e0d439fdb"
COLLECTION_RESOURCE = "54bf1022-a0b8-4f95-a5e9-82c084b2f533"  # arbitrary test value


def setUpModule():
    if not GraphModel.objects.filter(pk=PHYSICAL_THING_GRAPH_ID).exists():
        # TODO: this command isn't using a tempdir, so it's leaving behind files
        call_command("packages", "-o load_package -s arches_for_science/pkg -y".split())
    if not User.objects.filter(username="tester1").exists():
        call_command("add_test_users")


class AnalysisAreaAndSampleTakingTests(TestCase):
    def get_resource_instance(self, graph_id):
        r = ResourceInstance(graph_id=graph_id)
        r.save()  # not part of the transaction, part of the setup
        self.addCleanup(r.delete)
        return r

    def make_tile(self, parent_phys_thing, data, transaction_id):
        arbitrary_nodegroup = NodeGroup.objects.first()
        new_tile = TileModel(resourceinstance=parent_phys_thing, nodegroup_id=arbitrary_nodegroup.pk)
        new_tile.save()
        # Set the transactionid
        new_tile = Tile.objects.get(tileid=new_tile.pk)
        new_tile.data = data
        new_tile.save(transaction_id=transaction_id)

        return new_tile

    def test_create_delete_analysis_area(self):
        # TODO: fails with dev/dev? 🤔
        self.client.login(username="ben", password="Test12345!")

        transaction_id = "10000000-1000-1000-1000-100000000000"
        parent_phys_thing = self.get_resource_instance(PHYSICAL_THING_GRAPH_ID)
        create_data = {
            "transaction_id": transaction_id,  # NB: snake_case
            "parentPhysicalThingResourceid": str(parent_phys_thing.pk),  # NB: lowercase id
            "parentPhysicalThingName": "Test Name of Physical Thing",
            "collectionResourceid": COLLECTION_RESOURCE,
            "partIdentifierAssignmentTileData": JSONSerializer().serialize(
                {
                    PART_IDENTIFIER_ASSIGNMENT: [],
                    PART_IDENTIFIER_ASSIGNMENT_LABEL: {
                        get_language(): {"value": "test value", "direction": "ltr"},
                    },
                }
            ),
            "analysisAreaName": "Test Analysis Area",
        }
        response = self.client.post(reverse("saveanalysisarea"), create_data)
        self.assertEqual(response.status_code, 200)

        new_resource = ResourceInstance.objects.get(pk=response.json()["result"]["memberOfTile"]["resourceinstance_id"])
        new_tile_data = {PART_IDENTIFIER_ASSIGNMENT: [{"resourceId": str(new_resource.pk)}]}
        new_tile = self.make_tile(parent_phys_thing, new_tile_data, transaction_id)
        rxr = ResourceXResource(
            nodeid=Node.objects.get(pk=PART_IDENTIFIER_ASSIGNMENT),
            resourceinstanceidfrom=parent_phys_thing,
            resourceinstanceidto=new_resource,
            tileid=new_tile,
        )
        rxr.save()
        self.addCleanup(rxr.delete)

        delete_data = {
            "transactionId": transaction_id,  # NB: camelCase
            "parentPhysicalThingResourceId": str(parent_phys_thing.pk),  # NB: uppercase id
            "parentPhysicalThingTileData": JSONSerializer().serialize(new_tile_data),
        }
        delete_data = JSONSerializer().serialize(delete_data)
        content_type = "application/json"
        response = self.client.post(reverse("deleteanalysisarea"), delete_data, content_type=content_type)

        self.assertEqual(response.status_code, 200)

    def test_create_delete_sample(self):
        self.client.login(username="dev", password="dev")

        transaction_id = "10000000-1000-1000-1000-100000000001"
        parent_phys_thing = self.get_resource_instance(PHYSICAL_THING_GRAPH_ID)
        sampling_activity = self.get_resource_instance(SAMPLING_ACTIVITY_GRAPH_ID)
        part = self.get_resource_instance(PHYSICAL_THING_GRAPH_ID)

        # Create
        part_identifier_assignment_polygon_identifier_nodeid = "97c30c42-8594-11ea-97eb-acde48001122"
        part_identifier_assignment_tile_data = {
            PART_IDENTIFIER_ASSIGNMENT_LABEL: {get_language(): {"value": "test value", "direction": "ltr"}},
            PART_IDENTIFIER_ASSIGNMENT: [],
            part_identifier_assignment_polygon_identifier_nodeid: {},
        }

        create_data = {
            "transaction_id": transaction_id,  # NB: snake_case
            "parentPhysicalThingResourceid": str(parent_phys_thing.pk),
            "parentPhysicalThingName": "Test Name of Physical Thing",
            "collectionResourceid": COLLECTION_RESOURCE,
            "partIdentifierAssignmentTileData": JSONSerializer().serialize(part_identifier_assignment_tile_data),
            "partIdentifierAssignmentResourceId": str(part.pk),
            "sampleMotivation": "Test Motivation",
            "sampleDescription": "Test Description",
            "samplingActivityResourceId": str(sampling_activity.pk),
        }
        response = self.client.post(reverse("savesamplearea"), create_data)
        self.assertEqual(response.status_code, 200)

        # Delete
        result = response.json()["result"]
        physical_part_of_object_nodeid = PART_IDENTIFIER_ASSIGNMENT
        physical_part_of_object_resourceid = result["parentPhysicalThing"]["physicalPartOfObjectTile"]["data"][
            physical_part_of_object_nodeid
        ][0]["resourceId"]
        part_identifier_assignment_tile_data = {
            **part_identifier_assignment_tile_data,
            physical_part_of_object_nodeid: [
                {"resourceId": physical_part_of_object_resourceid},
            ],
        }

        delete_data = {
            "transactionId": transaction_id,  # NB: camelCase
            "parentPhysicalThingResourceid": str(parent_phys_thing.pk),
            "partIdentifierAssignmentTileData": JSONSerializer().serialize(part_identifier_assignment_tile_data),
            "samplingActivityResourceId": str(sampling_activity.pk),
            "collectionResourceid": COLLECTION_RESOURCE,
            "sampleMotivation": "Test Motivation",
            "sampleDescription": "Test Description",
        }
        delete_data = JSONSerializer().serialize(delete_data)
        content_type = "application/json"
        response = self.client.post(reverse("deletesamplearea"), delete_data, content_type=content_type)

        self.assertEqual(response.status_code, 200)



from arches.app.views.search import get_provisional_type
from django.views.generic import View
from arches.app.search.search_engine_factory import SearchEngineFactory
from django.http import HttpResponseNotFound
from django.utils.translation import gettext as _
from django.shortcuts import render
from datetime import datetime
from arches.app.search.components.base import SearchFilterFactory
from arches.app.search.elasticsearch_dsl_builder import Bool, Match, Query, Terms, MaxAgg, Aggregation, Nested
from arches.app.utils.permission_backend import get_nodegroups_by_perm
from arches.app.utils.response import JSONResponse
from arches.app.utils.betterJSONSerializer import JSONSerializer, JSONDeserializer
from arches.app.views.base import BaseManagerView
from arches.app.models import models
from arches.app.models.system_settings import settings


class PhysicalThingSearchView(View):
    def get(self, request):
        return self.search_results(request)

    def search_results(self, request):
        has_filters = len(list(request.GET.items())) > 1
        request.GET = request.GET.copy()

        se = SearchEngineFactory().create()
        search_results_object = {"query": Query(se)}

        include_provisional = get_provisional_type(request)
        permitted_nodegroups = self.get_permitted_nodegroups(request.user)

        # get a list of resourceIds of type Person et al that meet
        # the search query then use those ids to search for Person instances related to Physical Thing instances
        request.GET["resource-type-filter"] = '[{"graphid":"f71f7b9c-b25b-11e9-901e-a4d18cec433a","name":"Person","inverted":false}]'

        search_filter_factory = SearchFilterFactory(request)
        try:
            for filter_type, querystring in list(request.GET.items()) + [("search-results", "")]:
                search_filter = search_filter_factory.get_filter(filter_type)
                if search_filter:
                    search_filter.append_dsl(
                        search_results_object,
                        querystring=querystring,
                        permitted_nodegroups=permitted_nodegroups,
                        include_provisional=include_provisional,
                    )
        except Exception as err:
            return JSONResponse(err.message, status=500)

        dsl = search_results_object.pop("query", None)
        dsl.exclude("*")
        results = dsl.search(index="resources", limit=1000)
        resourceIds = [hit["_id"] for hit in results["hits"]["hits"]]

        search_results_object = {"query": Query(se)}

        request.GET["resource-type-filter"] = (
            '[{"graphid":"9519cb4f-b25b-11e9-8c7b-a4d18cec433a","name":"Physical Thing","inverted":false}]'
        )
        search_filter_factory = SearchFilterFactory(request)
        try:
            for filter_type, querystring in list(request.GET.items()) + [("search-results", "")]:
                search_filter = search_filter_factory.get_filter(filter_type)
                if search_filter:
                    search_filter.append_dsl(
                        search_results_object,
                        querystring=querystring,
                        permitted_nodegroups=permitted_nodegroups,
                        include_provisional=include_provisional,
                    )
        except Exception as err:
            return JSONResponse(err.message, status=500)

        # only search for realted instance references when a filter is applied (aside from the paging filter)
        if has_filters:
            # the resource id query can only return resources of type Physical Thing; this is to handle for the scenario
            # where a Person (or other relatable model added to filter) is related to something other than a Person
            resource_id_query = Bool()
            resource_id_query.must(Nested(path="ids", query=Terms(field="ids.id", terms=resourceIds)))
            resource_id_query.must(Terms(field="graph_id", terms="9519cb4f-b25b-11e9-8c7b-a4d18cec433a"))

            # we need to wrap the existing query (search_results_object['query']) in a Bool query along with
            # the resource id query above
            outer_query = Bool()
            outer_query.should(resource_id_query)

            dsl = search_results_object.pop("query", None)
            outer_query.should(dsl.dsl.pop("query", None))
            dsl.dsl["query"] = {}
            search_results_object["query"] = dsl

            search_results_object["query"].add_query(outer_query)

        dsl = search_results_object.pop("query", None)
        dsl.include("graph_id")
        dsl.include("root_ontology_class")
        dsl.include("resourceinstanceid")
        dsl.include("points")
        dsl.include("geometries")
        dsl.include("displayname")
        dsl.include("displaydescription")
        dsl.include("map_popup")
        dsl.include("provisional_resource")
        dsl.include("domains")
        # if request.GET.get('tiles', None) is not None:
        #     dsl.include('tiles')

        dsl.include("tiles")
        results = dsl.search(index="resources")
        # print JSONSerializer().serialize(dsl.dsl)

        if results is not None:
            # allow filters to modify the results
            for filter_type, querystring in list(request.GET.items()) + [("search-results", "")]:
                search_filter = search_filter_factory.get_filter(filter_type)
                if "results" not in results:
                    results = {"results": results}
                if search_filter:
                    search_filter.post_search_hook(search_results_object, results, permitted_nodegroups=permitted_nodegroups)

            ret = {}
            ret["results"] = results

            for key, value in list(search_results_object.items()):
                ret[key] = value

            ret["reviewer"] = request.user.groups.filter(name="Resource Reviewer").exists()
            ret["timestamp"] = datetime.now()
            ret["total_results"] = dsl.count(index="resources")

            return JSONResponse(ret)
        else:
            return HttpResponseNotFound(_("There was an error retrieving the search results"))

    def get_permitted_nodegroups(self, user):
        return [str(nodegroupid) for nodegroupid in get_nodegroups_by_perm(user, "models.read_nodegroup")]



CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_BEGIN = """
DECLARE
    fn_config jsonb;
    name_template text;
    map_popup_template text;
    description_template text;
    all_templates text[];
    this_template text;
    loop_index int;
    descriptor_key text;

    graph UUID;
    resourceid UUID;
    node_alias text;
    alias_with_separators text;
    node_for_alias UUID;
    nodegroup_for_alias UUID;
    localized_string_node_value jsonb;
    lang text;
    inner18n_obj_for_lang jsonb;
    resolved_node_value_for_lang text;
    resolved_node_values_by_descriptor text;
    working_string text;
    localized_result_all jsonb;
    localized_result_name_only jsonb;
    sample_localized_name jsonb;

BEGIN
"""

# For tiles trigger
CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_FIRST_BRANCH_SINGLE = """
SELECT NEW.resourceinstanceid INTO resourceid;
SELECT graphid INTO graph FROM resource_instances WHERE resourceinstanceid = resourceid;

SELECT config
INTO fn_config
FROM functions_x_graphs
WHERE
    functionid = '00b2d15a-fda0-4578-b79a-784e4138664b'
    AND functions_x_graphs.config IS NOT NULL
    AND graph = graphid;

IF FOUND THEN
"""

# For functions_x_graphs trigger
CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_FIRST_BRANCH_ALL = """
SELECT NEW.config, NEW.graphid
INTO fn_config, graph;

FOR resourceid IN (SELECT resourceinstanceid FROM resource_instances WHERE graphid = graph)
LOOP
"""

CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_COMMON = r"""
    SELECT JSONB_OBJECT('{}') INTO localized_result_all;
    SELECT JSONB_OBJECT('{}') INTO localized_result_name_only;
    SELECT fn_config -> 'descriptor_types' -> 'name' -> 'string_template' INTO name_template;
    SELECT fn_config -> 'descriptor_types' -> 'map_popup' -> 'string_template' INTO map_popup_template;
    SELECT fn_config -> 'descriptor_types' -> 'description' -> 'string_template' INTO description_template;

    SELECT ARRAY[name_template, map_popup_template, description_template] INTO all_templates;
    SELECT 0 INTO loop_index;
    FOR this_template IN SELECT UNNEST(all_templates)
    LOOP
        SELECT loop_index + 1 INTO loop_index;
        SELECT
        CASE
            WHEN loop_index = 1 THEN 'name'
            WHEN loop_index = 2 THEN 'map_popup'
            ELSE 'description'
        END
        INTO descriptor_key;

        -- Remove outer quotes from template
        SELECT TRIM(this_template, '"') INTO this_template;
        -- Unescape some characters before doing string replacement
        SELECT REPLACE(this_template, '\"', '"') INTO this_template;

        -- Resolve node values to localized strings
        FOR alias_with_separators IN SELECT UNNEST(REGEXP_MATCHES(this_template, '<\w+>', 'g'))
        LOOP
            SELECT TRIM(BOTH '<>' FROM alias_with_separators) INTO node_alias;

            SELECT nodeid, nodegroupid INTO node_for_alias, nodegroup_for_alias
                FROM nodes
                WHERE alias = node_alias
                AND graphid = graph;

            SELECT tiledata ->> node_for_alias::text INTO localized_string_node_value
                FROM tiles
                WHERE resourceinstanceid = resourceid
                AND nodegroupid = nodegroup_for_alias
                AND sortorder = 0
                LIMIT 1;

            -- Handle no value for string node

            IF localized_string_node_value IS NULL THEN
                -- Update the template (used below if no aliases have been resolved yet)
                SELECT REPLACE(this_template, alias_with_separators, ' -- ') INTO this_template;
                -- Update any intermediate values already saved
                FOR lang, resolved_node_values_by_descriptor IN SELECT * FROM jsonb_each(localized_result_all)
                LOOP
                    SELECT REPLACE(
                        TRIM((resolved_node_values_by_descriptor::jsonb -> descriptor_key)::text, '\"'),
                        alias_with_separators,
                        ' -- '
                    ) INTO resolved_node_value_for_lang;

                    IF resolved_node_value_for_lang IS NOT NULL THEN
                        SELECT jsonb_set(
                            localized_result_all,
                            ARRAY[lang, descriptor_key],
                            TO_JSONB(resolved_node_value_for_lang)
                        ) INTO localized_result_all;
                    END IF;
                END LOOP;

                IF descriptor_key = 'name' THEN
                    FOR lang, resolved_node_value_for_lang IN SELECT * FROM jsonb_each(localized_result_name_only)
                    LOOP
                        SELECT REPLACE(
                            TRIM(resolved_node_value_for_lang, '\"'),
                            alias_with_separators,
                            ' -- '
                        ) INTO resolved_node_value_for_lang;

                        IF resolved_node_value_for_lang IS NOT NULL THEN
                            SELECT jsonb_set(
                                localized_result_name_only,
                                ARRAY[lang],
                                TO_JSONB(resolved_node_value_for_lang)
                            ) INTO localized_result_name_only;
                        END IF;
                    END LOOP;
                END IF;

            END IF;

            -- Handle localized string value

            FOR lang, inner18n_obj_for_lang IN SELECT * FROM jsonb_each(localized_string_node_value) 
            LOOP
                -- Initialize language key if missing
                IF localized_result_all -> lang IS NULL THEN
                    SELECT jsonb_set(
                        localized_result_all,
                        ARRAY[lang],
                        JSONB_OBJECT('{}')
                    ) INTO localized_result_all;
                END IF;

                -- Retrieve the current working value, or start from the template
                SELECT
                CASE
                    WHEN localized_result_all -> lang -> descriptor_key IS NULL
                        THEN this_template
                    ELSE TRIM((localized_result_all -> lang -> descriptor_key)::text, '\"')
                END
                INTO working_string;

                SELECT REPLACE(
                    working_string,
                    alias_with_separators,
                    TRIM((inner18n_obj_for_lang -> 'value')::text, '"')
                )
                INTO resolved_node_value_for_lang;

                -- Update the working value
                SELECT jsonb_set(
                    localized_result_all,
                    ARRAY[lang, descriptor_key],
                    TO_JSONB(resolved_node_value_for_lang)
                ) INTO localized_result_all;

                IF descriptor_key = 'name' THEN
                    SELECT jsonb_set(
                        localized_result_name_only,
                        ARRAY[lang],
                        TO_JSONB(resolved_node_value_for_lang)
                    ) INTO localized_result_name_only;
                END IF;

            END LOOP;

        END LOOP;

        -- Handle a template having no dynamic values.
        IF alias_with_separators IS NULL THEN
            -- Get a list of languages from an existing descriptor
            SELECT name INTO sample_localized_name FROM resource_instances WHERE graphid = graph LIMIT 1;

            FOR lang IN SELECT * FROM JSONB_OBJECT_KEYS(sample_localized_name)
            LOOP
                SELECT JSONB_SET(
                    localized_result_all,
                    ARRAY[lang, descriptor_key],
                    TO_JSONB(this_template)
                ) INTO localized_result_all;
            END LOOP;
            IF descriptor_key = 'name' THEN
                FOR lang IN SELECT * FROM JSONB_OBJECT_KEYS(sample_localized_name)
                LOOP
                    SELECT JSONB_SET(
                        localized_result_name_only,
                        ARRAY[lang],
                        TO_JSONB(this_template)
                    ) INTO localized_result_name_only;
                END LOOP;
            END IF;
        END IF;

    END LOOP;

    UPDATE resource_instances
    SET descriptors = localized_result_all, name = localized_result_name_only
    WHERE resourceinstanceid = resourceid;

"""

CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_END = """
END IF;
END;
RETURN NULL;
"""

CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_SINGLE = (
    CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_BEGIN
    + CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_FIRST_BRANCH_SINGLE
    + CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_COMMON
    + CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_END
)


CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_ALL = (
    CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_BEGIN
    + CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_FIRST_BRANCH_ALL
    + CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_COMMON
    + CALCULATE_MULTICARD_PRIMARY_DESCRIPTOR_END.replace("IF", "LOOP")
)



# Generated by Django 4.2.4 on 2023-12-04 08:57

from django.db import migrations
import pgtrigger.compiler
import pgtrigger.migrations


class Migration(migrations.Migration):
    dependencies = [
        ("models", "10260_add_iiifmanifest_globalid"),
        ("arches_for_science", "0004_improve_primary_descriptor"),
    ]

    operations = [
        migrations.CreateModel(
            name="FunctionXGraphProxy",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("models.functionxgraph",),
        ),
        migrations.CreateModel(
            name="TileModelProxy",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=("models.tilemodel",),
        ),
        pgtrigger.migrations.AddTrigger(
            model_name="functionxgraphproxy",
            trigger=pgtrigger.compiler.Trigger(
                name="calculate_multicard_primary_descriptor_all",
                sql=pgtrigger.compiler.UpsertTriggerSql(
                    condition='WHEN (NEW."config" IS NOT NULL AND NEW."functionid" = \'00b2d15a-fda0-4578-b79a-784e4138664b\'::uuid)',
                    func="\nDECLARE\n    fn_config jsonb;\n    name_template text;\n    map_popup_template text;\n    description_template text;\n    all_templates text[];\n    this_template text;\n    loop_index int;\n    descriptor_key text;\n\n    graph UUID;\n    resourceid UUID;\n    node_alias text;\n    alias_with_separators text;\n    node_for_alias UUID;\n    nodegroup_for_alias UUID;\n    localized_string_node_value jsonb;\n    lang text;\n    inner18n_obj_for_lang jsonb;\n    resolved_node_value_for_lang text;\n    resolved_node_values_by_descriptor text;\n    working_string text;\n    localized_result_all jsonb;\n    localized_result_name_only jsonb;\n    sample_localized_name jsonb;\n\nBEGIN\n\nSELECT NEW.config, NEW.graphid\nINTO fn_config, graph;\n\nFOR resourceid IN (SELECT resourceinstanceid FROM resource_instances WHERE graphid = graph)\nLOOP\n\n    SELECT JSONB_OBJECT('{}') INTO localized_result_all;\n    SELECT JSONB_OBJECT('{}') INTO localized_result_name_only;\n    SELECT fn_config -> 'descriptor_types' -> 'name' -> 'string_template' INTO name_template;\n    SELECT fn_config -> 'descriptor_types' -> 'map_popup' -> 'string_template' INTO map_popup_template;\n    SELECT fn_config -> 'descriptor_types' -> 'description' -> 'string_template' INTO description_template;\n\n    SELECT ARRAY[name_template, map_popup_template, description_template] INTO all_templates;\n    SELECT 0 INTO loop_index;\n    FOR this_template IN SELECT UNNEST(all_templates)\n    LOOP\n        SELECT loop_index + 1 INTO loop_index;\n        SELECT\n        CASE\n            WHEN loop_index = 1 THEN 'name'\n            WHEN loop_index = 2 THEN 'map_popup'\n            ELSE 'description'\n        END\n        INTO descriptor_key;\n\n        -- Remove outer quotes from template\n        SELECT TRIM(this_template, '\"') INTO this_template;\n        -- Unescape some characters before doing string replacement\n        SELECT REPLACE(this_template, '\\\"', '\"') INTO this_template;\n\n        -- Resolve node values to localized strings\n        FOR alias_with_separators IN SELECT UNNEST(REGEXP_MATCHES(this_template, '<\\w+>', 'g'))\n        LOOP\n            SELECT TRIM(BOTH '<>' FROM alias_with_separators) INTO node_alias;\n\n            SELECT nodeid, nodegroupid INTO node_for_alias, nodegroup_for_alias\n                FROM nodes\n                WHERE alias = node_alias\n                AND graphid = graph;\n\n            SELECT tiledata ->> node_for_alias::text INTO localized_string_node_value\n                FROM tiles\n                WHERE resourceinstanceid = resourceid\n                AND nodegroupid = nodegroup_for_alias\n                AND sortorder = 0\n                LIMIT 1;\n\n            -- Handle no value for string node\n\n            IF localized_string_node_value IS NULL THEN\n                -- Update the template (used below if no aliases have been resolved yet)\n                SELECT REPLACE(this_template, alias_with_separators, ' -- ') INTO this_template;\n                -- Update any intermediate values already saved\n                FOR lang, resolved_node_values_by_descriptor IN SELECT * FROM jsonb_each(localized_result_all)\n                LOOP\n                    SELECT REPLACE(\n                        TRIM((resolved_node_values_by_descriptor::jsonb -> descriptor_key)::text, '\\\"'),\n                        alias_with_separators,\n                        ' -- '\n                    ) INTO resolved_node_value_for_lang;\n\n                    IF resolved_node_value_for_lang IS NOT NULL THEN\n                        SELECT jsonb_set(\n                            localized_result_all,\n                            ARRAY[lang, descriptor_key],\n                            TO_JSONB(resolved_node_value_for_lang)\n                        ) INTO localized_result_all;\n                    END IF;\n                END LOOP;\n\n                IF descriptor_key = 'name' THEN\n                    FOR lang, resolved_node_value_for_lang IN SELECT * FROM jsonb_each(localized_result_name_only)\n                    LOOP\n                        SELECT REPLACE(\n                            TRIM(resolved_node_value_for_lang, '\\\"'),\n                            alias_with_separators,\n                            ' -- '\n                        ) INTO resolved_node_value_for_lang;\n\n                        IF resolved_node_value_for_lang IS NOT NULL THEN\n                            SELECT jsonb_set(\n                                localized_result_name_only,\n                                ARRAY[lang],\n                                TO_JSONB(resolved_node_value_for_lang)\n                            ) INTO localized_result_name_only;\n                        END IF;\n                    END LOOP;\n                END IF;\n\n            END IF;\n\n            -- Handle localized string value\n\n            FOR lang, inner18n_obj_for_lang IN SELECT * FROM jsonb_each(localized_string_node_value) \n            LOOP\n                -- Initialize language key if missing\n                IF localized_result_all -> lang IS NULL THEN\n                    SELECT jsonb_set(\n                        localized_result_all,\n                        ARRAY[lang],\n                        JSONB_OBJECT('{}')\n                    ) INTO localized_result_all;\n                END IF;\n\n                -- Retrieve the current working value, or start from the template\n                SELECT\n                CASE\n                    WHEN localized_result_all -> lang -> descriptor_key IS NULL\n                        THEN this_template\n                    ELSE TRIM((localized_result_all -> lang -> descriptor_key)::text, '\\\"')\n                END\n                INTO working_string;\n\n                SELECT REPLACE(\n                    working_string,\n                    alias_with_separators,\n                    TRIM((inner18n_obj_for_lang -> 'value')::text, '\"')\n                )\n                INTO resolved_node_value_for_lang;\n\n                -- Update the working value\n                SELECT jsonb_set(\n                    localized_result_all,\n                    ARRAY[lang, descriptor_key],\n                    TO_JSONB(resolved_node_value_for_lang)\n                ) INTO localized_result_all;\n\n                IF descriptor_key = 'name' THEN\n                    SELECT jsonb_set(\n                        localized_result_name_only,\n                        ARRAY[lang],\n                        TO_JSONB(resolved_node_value_for_lang)\n                    ) INTO localized_result_name_only;\n                END IF;\n\n            END LOOP;\n\n        END LOOP;\n\n        -- Handle a template having no dynamic values.\n        IF alias_with_separators IS NULL THEN\n            -- Get a list of languages from an existing descriptor\n            SELECT name INTO sample_localized_name FROM resource_instances WHERE graphid = graph LIMIT 1;\n\n            FOR lang IN SELECT * FROM JSONB_OBJECT_KEYS(sample_localized_name)\n            LOOP\n                SELECT JSONB_SET(\n                    localized_result_all,\n                    ARRAY[lang, descriptor_key],\n                    TO_JSONB(this_template)\n                ) INTO localized_result_all;\n            END LOOP;\n            IF descriptor_key = 'name' THEN\n                FOR lang IN SELECT * FROM JSONB_OBJECT_KEYS(sample_localized_name)\n                LOOP\n                    SELECT JSONB_SET(\n                        localized_result_name_only,\n                        ARRAY[lang],\n                        TO_JSONB(this_template)\n                    ) INTO localized_result_name_only;\n                END LOOP;\n            END IF;\n        END IF;\n\n    END LOOP;\n\n    UPDATE resource_instances\n    SET descriptors = localized_result_all, name = localized_result_name_only\n    WHERE resourceinstanceid = resourceid;\n\n\nEND LOOP;\nEND;\nRETURN NULL;\n",
                    hash="d1df1edb9c1eff52c557852978f05b0e4b359610",
                    operation="INSERT OR UPDATE",
                    pgid="pgtrigger_calculate_multicard_primary_descriptor_all_3029e",
                    table="functions_x_graphs",
                    when="AFTER",
                ),
            ),
        ),
        pgtrigger.migrations.AddTrigger(
            model_name="tilemodelproxy",
            trigger=pgtrigger.compiler.Trigger(
                name="calculate_multicard_primary_descriptor_single",
                sql=pgtrigger.compiler.UpsertTriggerSql(
                    func="\nDECLARE\n    fn_config jsonb;\n    name_template text;\n    map_popup_template text;\n    description_template text;\n    all_templates text[];\n    this_template text;\n    loop_index int;\n    descriptor_key text;\n\n    graph UUID;\n    resourceid UUID;\n    node_alias text;\n    alias_with_separators text;\n    node_for_alias UUID;\n    nodegroup_for_alias UUID;\n    localized_string_node_value jsonb;\n    lang text;\n    inner18n_obj_for_lang jsonb;\n    resolved_node_value_for_lang text;\n    resolved_node_values_by_descriptor text;\n    working_string text;\n    localized_result_all jsonb;\n    localized_result_name_only jsonb;\n    sample_localized_name jsonb;\n\nBEGIN\n\nSELECT NEW.resourceinstanceid INTO resourceid;\nSELECT graphid INTO graph FROM resource_instances WHERE resourceinstanceid = resourceid;\n\nSELECT config\nINTO fn_config\nFROM functions_x_graphs\nWHERE\n    functionid = '00b2d15a-fda0-4578-b79a-784e4138664b'\n    AND functions_x_graphs.config IS NOT NULL\n    AND graph = graphid;\n\nIF FOUND THEN\n\n    SELECT JSONB_OBJECT('{}') INTO localized_result_all;\n    SELECT JSONB_OBJECT('{}') INTO localized_result_name_only;\n    SELECT fn_config -> 'descriptor_types' -> 'name' -> 'string_template' INTO name_template;\n    SELECT fn_config -> 'descriptor_types' -> 'map_popup' -> 'string_template' INTO map_popup_template;\n    SELECT fn_config -> 'descriptor_types' -> 'description' -> 'string_template' INTO description_template;\n\n    SELECT ARRAY[name_template, map_popup_template, description_template] INTO all_templates;\n    SELECT 0 INTO loop_index;\n    FOR this_template IN SELECT UNNEST(all_templates)\n    LOOP\n        SELECT loop_index + 1 INTO loop_index;\n        SELECT\n        CASE\n            WHEN loop_index = 1 THEN 'name'\n            WHEN loop_index = 2 THEN 'map_popup'\n            ELSE 'description'\n        END\n        INTO descriptor_key;\n\n        -- Remove outer quotes from template\n        SELECT TRIM(this_template, '\"') INTO this_template;\n        -- Unescape some characters before doing string replacement\n        SELECT REPLACE(this_template, '\\\"', '\"') INTO this_template;\n\n        -- Resolve node values to localized strings\n        FOR alias_with_separators IN SELECT UNNEST(REGEXP_MATCHES(this_template, '<\\w+>', 'g'))\n        LOOP\n            SELECT TRIM(BOTH '<>' FROM alias_with_separators) INTO node_alias;\n\n            SELECT nodeid, nodegroupid INTO node_for_alias, nodegroup_for_alias\n                FROM nodes\n                WHERE alias = node_alias\n                AND graphid = graph;\n\n            SELECT tiledata ->> node_for_alias::text INTO localized_string_node_value\n                FROM tiles\n                WHERE resourceinstanceid = resourceid\n                AND nodegroupid = nodegroup_for_alias\n                AND sortorder = 0\n                LIMIT 1;\n\n            -- Handle no value for string node\n\n            IF localized_string_node_value IS NULL THEN\n                -- Update the template (used below if no aliases have been resolved yet)\n                SELECT REPLACE(this_template, alias_with_separators, ' -- ') INTO this_template;\n                -- Update any intermediate values already saved\n                FOR lang, resolved_node_values_by_descriptor IN SELECT * FROM jsonb_each(localized_result_all)\n                LOOP\n                    SELECT REPLACE(\n                        TRIM((resolved_node_values_by_descriptor::jsonb -> descriptor_key)::text, '\\\"'),\n                        alias_with_separators,\n                        ' -- '\n                    ) INTO resolved_node_value_for_lang;\n\n                    IF resolved_node_value_for_lang IS NOT NULL THEN\n                        SELECT jsonb_set(\n                            localized_result_all,\n                            ARRAY[lang, descriptor_key],\n                            TO_JSONB(resolved_node_value_for_lang)\n                        ) INTO localized_result_all;\n                    END IF;\n                END LOOP;\n\n                IF descriptor_key = 'name' THEN\n                    FOR lang, resolved_node_value_for_lang IN SELECT * FROM jsonb_each(localized_result_name_only)\n                    LOOP\n                        SELECT REPLACE(\n                            TRIM(resolved_node_value_for_lang, '\\\"'),\n                            alias_with_separators,\n                            ' -- '\n                        ) INTO resolved_node_value_for_lang;\n\n                        IF resolved_node_value_for_lang IS NOT NULL THEN\n                            SELECT jsonb_set(\n                                localized_result_name_only,\n                                ARRAY[lang],\n                                TO_JSONB(resolved_node_value_for_lang)\n                            ) INTO localized_result_name_only;\n                        END IF;\n                    END LOOP;\n                END IF;\n\n            END IF;\n\n            -- Handle localized string value\n\n            FOR lang, inner18n_obj_for_lang IN SELECT * FROM jsonb_each(localized_string_node_value) \n            LOOP\n                -- Initialize language key if missing\n                IF localized_result_all -> lang IS NULL THEN\n                    SELECT jsonb_set(\n                        localized_result_all,\n                        ARRAY[lang],\n                        JSONB_OBJECT('{}')\n                    ) INTO localized_result_all;\n                END IF;\n\n                -- Retrieve the current working value, or start from the template\n                SELECT\n                CASE\n                    WHEN localized_result_all -> lang -> descriptor_key IS NULL\n                        THEN this_template\n                    ELSE TRIM((localized_result_all -> lang -> descriptor_key)::text, '\\\"')\n                END\n                INTO working_string;\n\n                SELECT REPLACE(\n                    working_string,\n                    alias_with_separators,\n                    TRIM((inner18n_obj_for_lang -> 'value')::text, '\"')\n                )\n                INTO resolved_node_value_for_lang;\n\n                -- Update the working value\n                SELECT jsonb_set(\n                    localized_result_all,\n                    ARRAY[lang, descriptor_key],\n                    TO_JSONB(resolved_node_value_for_lang)\n                ) INTO localized_result_all;\n\n                IF descriptor_key = 'name' THEN\n                    SELECT jsonb_set(\n                        localized_result_name_only,\n                        ARRAY[lang],\n                        TO_JSONB(resolved_node_value_for_lang)\n                    ) INTO localized_result_name_only;\n                END IF;\n\n            END LOOP;\n\n        END LOOP;\n\n        -- Handle a template having no dynamic values.\n        IF alias_with_separators IS NULL THEN\n            -- Get a list of languages from an existing descriptor\n            SELECT name INTO sample_localized_name FROM resource_instances WHERE graphid = graph LIMIT 1;\n\n            FOR lang IN SELECT * FROM JSONB_OBJECT_KEYS(sample_localized_name)\n            LOOP\n                SELECT JSONB_SET(\n                    localized_result_all,\n                    ARRAY[lang, descriptor_key],\n                    TO_JSONB(this_template)\n                ) INTO localized_result_all;\n            END LOOP;\n            IF descriptor_key = 'name' THEN\n                FOR lang IN SELECT * FROM JSONB_OBJECT_KEYS(sample_localized_name)\n                LOOP\n                    SELECT JSONB_SET(\n                        localized_result_name_only,\n                        ARRAY[lang],\n                        TO_JSONB(this_template)\n                    ) INTO localized_result_name_only;\n                END LOOP;\n            END IF;\n        END IF;\n\n    END LOOP;\n\n    UPDATE resource_instances\n    SET descriptors = localized_result_all, name = localized_result_name_only\n    WHERE resourceinstanceid = resourceid;\n\n\nEND IF;\nEND;\nRETURN NULL;\n",
                    hash="2bc90a07fa285aa4dd196e208cc1c1ffa2bb45eb",
                    operation="INSERT OR UPDATE OR DELETE",
                    pgid="pgtrigger_calculate_multicard_primary_descriptor_single_26494",
                    table="tiles",
                    when="AFTER",
                ),
            ),
        ),
    ]



"""
ARCHES - a program developed to inventory and manage immovable cultural heritage.
Copyright (C) 2013 J. Paul Getty Trust and World Monuments Fund

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from arches.settings import *
from arches_for_science.settings import *
import arches
import os

try:
    from django.utils.translation import gettext_lazy as _
except ImportError:  # unable to import prior to installing requirements
    pass

APP_ROOT = os.path.dirname(__file__)
TEST_ROOT = os.path.normpath(os.path.join(ROOT_DIR, "..", "tests"))


MIN_ARCHES_VERSION = arches.__version__
MAX_ARCHES_VERSION = arches.__version__

# LOAD_V3_DATA_DURING_TESTS = True will engage the most extensive the of the v3
# data migration tests, which could add over a minute to the test process. It's
# recommended that this setting only be set to True in tests/settings_local.py
# and run in specific cases at the discretion of the developer.
LOAD_V3_DATA_DURING_TESTS = False

RESOURCE_GRAPH_LOCATIONS = (os.path.join(TEST_ROOT, "fixtures", "resource_graphs"),)

ONTOLOGY_FIXTURES = os.path.join(TEST_ROOT, "fixtures", "ontologies", "test_ontology")
ONTOLOGY_PATH = os.path.join(TEST_ROOT, "fixtures", "ontologies", "cidoc_crm")
MEDIA_ROOT = os.path.join(TEST_ROOT, "fixtures", "data")

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "arches",
    "arches.app.models",
    "arches.management",
    "guardian",
    "oauth2_provider",
    "pgtrigger",
    "arches_templating",
    "arches_for_science",
]

DATABASES = {
    "default": {
        "ATOMIC_REQUESTS": False,
        "AUTOCOMMIT": True,
        "CONN_MAX_AGE": 0,
        "ENGINE": "django.contrib.gis.db.backends.postgis",
        "HOST": "localhost",
        "NAME": "arches_for_science",
        "OPTIONS": {},
        "PASSWORD": "postgis",
        "PORT": "5432",
        "POSTGIS_TEMPLATE": "template_postgis",
        "TEST": {"CHARSET": None, "COLLATION": None, "MIRROR": None, "NAME": None},
        "TIME_ZONE": None,
        "USER": "postgres",
    }
}

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
    },
    "user_permission": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
        "LOCATION": "user_permission_cache",
    },
}

LOGGING["loggers"]["arches"]["level"] = "ERROR"

ELASTICSEARCH_PREFIX = "test"

TEST_RUNNER = "arches.test.runner.ArchesTestRunner"
SILENCED_SYSTEM_CHECKS.append("arches.W001")  # Cache backend does not support rate-limiting

# could add Chrome, PhantomJS etc... here
LOCAL_BROWSERS = []  # ['Firefox']

ENABLE_USER_SIGNUP = True
FORCE_USER_SIGNUP_EMAIL_AUTHENTICATION = True

OVERRIDE_RESOURCE_MODEL_LOCK = True

ENABLE_TWO_FACTOR_AUTHENTICATION = False
FORCE_TWO_FACTOR_AUTHENTICATION = False

DATATYPE_LOCATIONS.append("tests.fixtures.datatypes")
ELASTICSEARCH_HOSTS = [{"scheme": "http", "host": "localhost", "port": ELASTICSEARCH_HTTP_PORT}]
LANGUAGES = [
    ("de", _("German")),
    ("en", _("English")),
    ("en-gb", _("British English")),
    ("es", _("Spanish")),
    ("ar", _("Arabic")),
]

DOCKER = False



from django.views.generic import View
from arches.app.utils.response import JSONResponse
from arches.app.models.models import TileModel
from arches.app.models.resource import Resource
from django.core.exceptions import ObjectDoesNotExist


class DigitalResourcesByObjectParts(View):
    def get(self, request, resourceid):
        parts_nodegroup_id = "fec59582-8593-11ea-97eb-acde48001122"
        physical_thing_node_id = "b240c366-8594-11ea-97eb-acde48001122"
        digital_resource_nodegroup_id = "8a4ad932-8d59-11eb-a9c4-faffc265b501"
        digital_resource_node_id = "a298ee52-8d59-11eb-a9c4-faffc265b501"
        digital_resource_type_nodegroup_id = "09c1778a-ca7b-11e9-860b-a4d18cec433a"
        manifest_concept_value_id = "305c62f0-7e3d-4d52-a210-b451491e6100"
        part_tiles = TileModel.objects.filter(resourceinstance_id=resourceid).filter(nodegroup_id=parts_nodegroup_id)
        physical_things = [tile.data[physical_thing_node_id][0]["resourceId"] for tile in part_tiles]
        physical_things.append(resourceid)
        digital_resource_tiles = TileModel.objects.filter(resourceinstance_id__in=physical_things).filter(
            nodegroup_id=digital_resource_nodegroup_id
        )
        related_datasets = {tile.data[digital_resource_node_id][0]["resourceId"] for tile in digital_resource_tiles}
        part_lookup = {
            tile.data[digital_resource_node_id][0]["resourceId"]: str(tile.resourceinstance_id) for tile in digital_resource_tiles
        }
        digital_resource_type_tiles = TileModel.objects.filter(resourceinstance_id__in=related_datasets).filter(
            nodegroup_id=digital_resource_type_nodegroup_id
        )
        manifest_datasets = {
            str(tile.resourceinstance_id)
            for tile in digital_resource_type_tiles
            if manifest_concept_value_id in tile.data[digital_resource_type_nodegroup_id]
        }
        digital_resources = Resource.objects.filter(pk__in=related_datasets - manifest_datasets)
        object_type = self.get_type(resourceid)
        results = {
            "resources": [{"resourceid": str(resource.pk), "displayname": resource.displayname()} for resource in digital_resources],
            "selected_resource": resourceid,
            "type": object_type,
        }
        for resource in results["resources"]:
            resource["partresourceid"] = part_lookup[resource["resourceid"]]
            resource["isdirect"] = part_lookup[resource["resourceid"]] == resourceid
        return JSONResponse(results)

    def get_type(self, resourceid):
        type_nodegroup_id = "8ddfe3ab-b31d-11e9-aff0-a4d18cec433a"
        sample_area_value_id = "7375a6fb-0bfb-4bcf-81a3-6180cdd26123"
        sample_value_id = "77d8cf19-ce9c-4e0a-bde1-9148d870e11c"
        analysis_area_value_id = "31d97bdd-f10f-4a26-958c-69cb5ab69af1"

        try:
            object_type_tile = TileModel.objects.filter(resourceinstance_id=resourceid).get(nodegroup_id=type_nodegroup_id)
            object_types = object_type_tile.data[type_nodegroup_id]
            if sample_area_value_id in object_types:
                return "sample area"
            elif sample_value_id in object_types:
                return "sample"
            elif analysis_area_value_id in object_types:
                return "analysis area"
            else:
                return ""
        except (ObjectDoesNotExist, KeyError) as e:
            return ""



from django.db import migrations
from django.db import connection


class Migration(migrations.Migration):

    dependencies = [
        ("arches_for_science", "0006_remove_TopConcept"),
    ]

    def forward(apps, schema_editor):
        sql = """
            -- Add 2D Data Measurement
            insert into concepts (conceptid, legacyoid, nodetype) values ('c6ad85c9-12a5-40f5-8773-9b513ded7585', 'https://afs.test.fargeo.com/c6ad85c9-12a5-40f5-8773-9b513ded7585', 'Concept');
            insert into values (valueid, value, conceptid, languageid, valuetype) values ('1b23b170-114c-4602-b01a-4d0f8aba05ec', '2D Data Measurement', 'c6ad85c9-12a5-40f5-8773-9b513ded7585', 'en', 'prefLabel');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('4157e908-660c-45cc-a20f-06a3841b1543', 'c6ad85c9-12a5-40f5-8773-9b513ded7585','narrower');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('0ca18480-f5f2-41b2-b177-857726e5ecd7', 'c6ad85c9-12a5-40f5-8773-9b513ded7585', 'member');
            -- Add 3D Data Measurement
            insert into concepts (conceptid, legacyoid, nodetype) values ('9f336e9a-17bd-45bc-a4a2-f816a551d196', 'https://afs.test.fargeo.com/9f336e9a-17bd-45bc-a4a2-f816a551d196', 'Concept');
            insert into values (valueid, value, conceptid, languageid, valuetype) values ('a1637323-dd13-41bb-98f1-80f963c6e0ea', '3D Data Measurement', '9f336e9a-17bd-45bc-a4a2-f816a551d196', 'en', 'prefLabel');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('4157e908-660c-45cc-a20f-06a3841b1543', '9f336e9a-17bd-45bc-a4a2-f816a551d196','narrower');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('0ca18480-f5f2-41b2-b177-857726e5ecd7', '9f336e9a-17bd-45bc-a4a2-f816a551d196', 'member');
            -- Move FORS, RAMAN, XRF to 2d
            update relations set conceptidfrom = 'c6ad85c9-12a5-40f5-8773-9b513ded7585' where conceptidfrom = '4157e908-660c-45cc-a20f-06a3841b1543' and conceptidto in (
                '977c662f-c1d9-4a35-b556-696517e312ec',
                'b0f10e3b-6dc6-41a1-b3ac-204ba3f9b982',
                '2e3e64b3-c802-4bbc-b284-5dbcd50ee4da'
            );
            update relations set conceptidfrom = 'c6ad85c9-12a5-40f5-8773-9b513ded7585' where conceptidfrom = '0ca18480-f5f2-41b2-b177-857726e5ecd7' and conceptidto in (
                '977c662f-c1d9-4a35-b556-696517e312ec',
                'b0f10e3b-6dc6-41a1-b3ac-204ba3f9b982',
                '2e3e64b3-c802-4bbc-b284-5dbcd50ee4da'
            ) and relationtype = 'member';

            update values set value = 'Point XRF Measurement' where conceptid = '977c662f-c1d9-4a35-b556-696517e312ec'; 
            insert into values (value, conceptid, languageid, valuetype) values ('Point X-ray fluorescence spectroscopy (XRF) Measurement', '977c662f-c1d9-4a35-b556-696517e312ec', 'en', 'altLabel');

            update values set value = 'Point Raman Measurement' where conceptid = 'b0f10e3b-6dc6-41a1-b3ac-204ba3f9b982';
            insert into values (value, conceptid, languageid, valuetype) values ('Point Raman spectroscopy Measurement', 'b0f10e3b-6dc6-41a1-b3ac-204ba3f9b982', 'en', 'altLabel');

            update values set value = 'FORS Measurement' where conceptid = '2e3e64b3-c802-4bbc-b284-5dbcd50ee4da';
            insert into values (value, conceptid, languageid, valuetype) values ('Fiber Optics Reflectance spectroscopy (FORS) Measurement', '2e3e64b3-c802-4bbc-b284-5dbcd50ee4da', 'en', 'altLabel');


            -- add new concepts
            --FTIR Measurement (Alt label*: Fourier-transform infrared (FTIR) spectroscopy Measurement) 9ef07691-f751-415a-805e-6fc52fbdb5a1
            insert into concepts (conceptid, legacyoid, nodetype) values ('9ef07691-f751-415a-805e-6fc52fbdb5a1', 'https://afs.test.fargeo.com/9ef07691-f751-415a-805e-6fc52fbdb5a1', 'Concept');
            insert into values (valueid, value, conceptid, languageid, valuetype) values ('851414d5-4a7f-4e70-a4fb-6d37e6657798', 'FTIR Measurement', '9ef07691-f751-415a-805e-6fc52fbdb5a1', 'en', 'prefLabel');
            insert into values (value, conceptid, languageid, valuetype) values ('Fourier-transform infrared (FTIR) spectroscopy Measurement', '9ef07691-f751-415a-805e-6fc52fbdb5a1', 'en', 'altLabel');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('c6ad85c9-12a5-40f5-8773-9b513ded7585', '9ef07691-f751-415a-805e-6fc52fbdb5a1', 'narrower');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('c6ad85c9-12a5-40f5-8773-9b513ded7585', '9ef07691-f751-415a-805e-6fc52fbdb5a1', 'member');

            --3D XRF Measurement (Alt label*: 3D X-ray fluorescence (XRF) spectroscopy Measurement) 091a244e-6532-4fc7-8b70-63364c4a4569
            insert into concepts (conceptid, legacyoid, nodetype) values ('091a244e-6532-4fc7-8b70-63364c4a4569', 'https://afs.test.fargeo.com/091a244e-6532-4fc7-8b70-63364c4a4569', 'Concept');
            insert into values (valueid, value, conceptid, languageid, valuetype) values ('17de9b07-66e5-4c97-a968-224fd31506e7', '3D XRF Measurement', '091a244e-6532-4fc7-8b70-63364c4a4569', 'en', 'prefLabel');
            insert into values (value, conceptid, languageid, valuetype) values ('3D X-ray fluorescence (XRF) spectroscopy Measurement', '091a244e-6532-4fc7-8b70-63364c4a4569', 'en', 'altLabel');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('9f336e9a-17bd-45bc-a4a2-f816a551d196', '091a244e-6532-4fc7-8b70-63364c4a4569', 'narrower');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('9f336e9a-17bd-45bc-a4a2-f816a551d196', '091a244e-6532-4fc7-8b70-63364c4a4569', 'member');

            --RIS Measurement (Alt label*: Reflectance imaging spectroscopy (RIS) Measurement) d5c7a90b-fd87-4d2c-86f8-01909795e73e
            insert into concepts (conceptid, legacyoid, nodetype) values ('d5c7a90b-fd87-4d2c-86f8-01909795e73e', 'https://afs.test.fargeo.com/d5c7a90b-fd87-4d2c-86f8-01909795e73e', 'Concept');
            insert into values (valueid, value, conceptid, languageid, valuetype) values ('3e40bee5-736b-42b6-a51b-c7a5cc143b80', 'RIS Measurement', 'd5c7a90b-fd87-4d2c-86f8-01909795e73e', 'en', 'prefLabel');
            insert into values (value, conceptid, languageid, valuetype) values ('Reflectance imaging spectroscopy (RIS) Measurement', 'd5c7a90b-fd87-4d2c-86f8-01909795e73e', 'en', 'altLabel');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('9f336e9a-17bd-45bc-a4a2-f816a551d196', 'd5c7a90b-fd87-4d2c-86f8-01909795e73e', 'narrower');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('9f336e9a-17bd-45bc-a4a2-f816a551d196', 'd5c7a90b-fd87-4d2c-86f8-01909795e73e', 'member');

            --SEM-EDS Measurement (Alt label*: Scanning electron microscopy – energy dispersive (SEM-EDS) spectroscopy Measurement) 9b1c5d1b-7ff9-400a-acde-c9a081eff7e1
            insert into concepts (conceptid, legacyoid, nodetype) values ('9b1c5d1b-7ff9-400a-acde-c9a081eff7e1', 'https://afs.test.fargeo.com/9b1c5d1b-7ff9-400a-acde-c9a081eff7e1', 'Concept');
            insert into values (valueid, value, conceptid, languageid, valuetype) values ('a1f180e3-96a7-46b8-beb5-beedfe94be86', 'SEM-EDS Measurement', '9b1c5d1b-7ff9-400a-acde-c9a081eff7e1', 'en', 'prefLabel');
            insert into values (value, conceptid, languageid, valuetype) values ('Scanning electron microscopy – energy dispersive (SEM-EDS) spectroscopy Measurement', '9b1c5d1b-7ff9-400a-acde-c9a081eff7e1', 'en', 'altLabel');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('9f336e9a-17bd-45bc-a4a2-f816a551d196', '9b1c5d1b-7ff9-400a-acde-c9a081eff7e1', 'narrower');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('9f336e9a-17bd-45bc-a4a2-f816a551d196', '9b1c5d1b-7ff9-400a-acde-c9a081eff7e1', 'member');

            -- 3D Raman Measurement (Alt label*: 3D Raman spectroscopy Measurement) fa1d6a48-4e82-4d23-8747-c4fa10f3301a
            insert into concepts (conceptid, legacyoid, nodetype) values ('fa1d6a48-4e82-4d23-8747-c4fa10f3301a', 'https://afs.test.fargeo.com/fa1d6a48-4e82-4d23-8747-c4fa10f3301a', 'Concept');
            insert into values (valueid, value, conceptid, languageid, valuetype) values ('fb0bb706-b082-4387-85cf-5ca59b3259bc', '3D Raman Measurement', 'fa1d6a48-4e82-4d23-8747-c4fa10f3301a', 'en', 'prefLabel');
            insert into values (value, conceptid, languageid, valuetype) values ('3D Raman spectroscopy Measurement', 'fa1d6a48-4e82-4d23-8747-c4fa10f3301a', 'en', 'altLabel');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('9f336e9a-17bd-45bc-a4a2-f816a551d196', 'fa1d6a48-4e82-4d23-8747-c4fa10f3301a', 'narrower');
            insert into relations (conceptidfrom, conceptidto, relationtype) values ('9f336e9a-17bd-45bc-a4a2-f816a551d196', 'fa1d6a48-4e82-4d23-8747-c4fa10f3301a', 'member');
        """

        with connection.cursor() as cursor:
            cursor.execute("""SELECT EXISTS (SELECT FROM information_schema.tables WHERE table_name = 'concepts');""")
            row = cursor.fetchone()
            if len(row) and row[0]:
                cursor.execute("""select * from concepts where conceptid = '4157e908-660c-45cc-a20f-06a3841b1543';""")
                row = cursor.fetchone()
                if row:
                    cursor.execute(sql)

    def reverse(apps, schema_editor):
        sql = """
            update values set value = 'XRF measurement' where conceptid = '977c662f-c1d9-4a35-b556-696517e312ec'; 
            update values set value = 'Raman measurement' where conceptid = 'b0f10e3b-6dc6-41a1-b3ac-204ba3f9b982';
            update values set value = 'FORS measurement' where conceptid = '2e3e64b3-c802-4bbc-b284-5dbcd50ee4da';
            update relations set conceptidfrom = '4157e908-660c-45cc-a20f-06a3841b1543' where conceptidfrom = 'c6ad85c9-12a5-40f5-8773-9b513ded7585' and conceptidto in (
                '977c662f-c1d9-4a35-b556-696517e312ec',
                'b0f10e3b-6dc6-41a1-b3ac-204ba3f9b982',
                '2e3e64b3-c802-4bbc-b284-5dbcd50ee4da'
            ) and relationtype = 'narrower';
            update relations set conceptidfrom = '0ca18480-f5f2-41b2-b177-857726e5ecd7' where conceptidfrom = 'c6ad85c9-12a5-40f5-8773-9b513ded7585' and conceptidto in (
                '977c662f-c1d9-4a35-b556-696517e312ec',
                'b0f10e3b-6dc6-41a1-b3ac-204ba3f9b982',
                '2e3e64b3-c802-4bbc-b284-5dbcd50ee4da'
            ) and relationtype = 'member';
            delete from values where conceptid = 'c6ad85c9-12a5-40f5-8773-9b513ded7585';
            delete from values where conceptid = '9f336e9a-17bd-45bc-a4a2-f816a551d196';
            delete from relations where conceptidto = 'c6ad85c9-12a5-40f5-8773-9b513ded7585' and conceptidfrom = '4157e908-660c-45cc-a20f-06a3841b1543'; -- delete 2d relation to obs
            delete from relations where conceptidto = '9f336e9a-17bd-45bc-a4a2-f816a551d196' and conceptidfrom = '4157e908-660c-45cc-a20f-06a3841b1543'; -- delete 3d relation to obs
            delete from values where valueid = '1b23b170-114c-4602-b01a-4d0f8aba05ec'; -- delete 2d data meas value
            delete from values where valueid = 'a1637323-dd13-41bb-98f1-80f963c6e0ea'; -- delete 3d data meas value
            delete from concepts where conceptid = 'c6ad85c9-12a5-40f5-8773-9b513ded7585'; -- delete 2d data meas concet
            delete from concepts where conceptid = '9f336e9a-17bd-45bc-a4a2-f816a551d196'; -- delete 3d data meas concept

            -- FTIR Measurement (Alt label*: Fourier-transform infrared (FTIR) spectroscopy Measurement) 9ef07691-f751-415a-805e-6fc52fbdb5a1
            delete from relations where conceptidto = '9ef07691-f751-415a-805e-6fc52fbdb5a1';
            delete from values where conceptid = '9ef07691-f751-415a-805e-6fc52fbdb5a1';
            delete from concepts where conceptid = '9ef07691-f751-415a-805e-6fc52fbdb5a1';

            --3D XRF Measurement (Alt label*: 3D X-ray fluorescence (XRF) spectroscopy Measurement) 091a244e-6532-4fc7-8b70-63364c4a4569
            delete from relations where conceptidto = '091a244e-6532-4fc7-8b70-63364c4a4569';
            delete from values where conceptid = '091a244e-6532-4fc7-8b70-63364c4a4569';
            delete from concepts where conceptid = '091a244e-6532-4fc7-8b70-63364c4a4569';

            --RIS Measurement (Alt label*: Reflectance imaging spectroscopy (RIS) Measurement) d5c7a90b-fd87-4d2c-86f8-01909795e73e
            delete from relations where conceptidto = 'd5c7a90b-fd87-4d2c-86f8-01909795e73e';
            delete from values where conceptid = 'd5c7a90b-fd87-4d2c-86f8-01909795e73e';
            delete from concepts where conceptid = 'd5c7a90b-fd87-4d2c-86f8-01909795e73e';

            --SEM-EDS Measurement (Alt label*: Scanning electron microscopy – energy dispersive (SEM-EDS) spectroscopy Measurement) 9b1c5d1b-7ff9-400a-acde-c9a081eff7e1
            delete from relations where conceptidto = '9b1c5d1b-7ff9-400a-acde-c9a081eff7e1';
            delete from values where conceptid = '9b1c5d1b-7ff9-400a-acde-c9a081eff7e1';
            delete from concepts where conceptid = '9b1c5d1b-7ff9-400a-acde-c9a081eff7e1';

            -- 3D Raman Measurement (Alt label*: 3D Raman spectroscopy Measurement) fa1d6a48-4e82-4d23-8747-c4fa10f3301a
            delete from relations where conceptidto = 'fa1d6a48-4e82-4d23-8747-c4fa10f3301a';
            delete from values where conceptid = 'fa1d6a48-4e82-4d23-8747-c4fa10f3301a';
            delete from concepts where conceptid = 'fa1d6a48-4e82-4d23-8747-c4fa10f3301a';

            -- reverse collection changes
            delete from relations where conceptidto = 'c6ad85c9-12a5-40f5-8773-9b513ded7585' and conceptidfrom = '0ca18480-f5f2-41b2-b177-857726e5ecd7'; -- delete 2d relation to obs member
            delete from relations where conceptidto = '9f336e9a-17bd-45bc-a4a2-f816a551d196' and conceptidfrom = '0ca18480-f5f2-41b2-b177-857726e5ecd7'; -- delete 3d relation to obs member


            delete from values where conceptid in (
                '977c662f-c1d9-4a35-b556-696517e312ec',
                'b0f10e3b-6dc6-41a1-b3ac-204ba3f9b982',
                '2e3e64b3-c802-4bbc-b284-5dbcd50ee4da'
            ) and valuetype = 'altLabel';
        """

        with connection.cursor() as cursor:
            cursor.execute(sql)

    operations = [migrations.RunPython(forward, reverse)]



import logging
from django.core.exceptions import ObjectDoesNotExist
from django.db import transaction
from django.utils.translation import gettext as _
from django.views.generic import View
from arches.app.datatypes.datatypes import StringDataType, DataTypeFactory
from arches.app.models.tile import Tile
from arches.app.models.models import TileModel, Node
from arches.app.models.resource import Resource
from arches.app.search.search_engine_factory import SearchEngineInstance as se
from arches.app.search.mappings import TERMS_INDEX, RESOURCES_INDEX
from arches.app.utils.betterJSONSerializer import JSONDeserializer
from arches.app.utils.response import JSONResponse

logger = logging.getLogger(__name__)
related_resource_template = {
    "resourceId": "",
    "ontologyProperty": "",
    "resourceXresourceId": "",
    "inverseOntologyProperty": "",
}


class UpdateResourceListView(View):
    """
    This view
    1. assumes one project resource, one collection resource, multiple physical thing resources
    2. if user send a collection resourceid, add or remove that resource id to the physical things in the data
    3. if user does not send any collection resourceid,
        a. create a collection instance
        b. add the collection to the project and
        c. add/remove the tile of the related collection to/from physical things (member_of_set_nodegroup_id)

    The request body should look like this
    {
        projectresourceid: always present but does not have to be used,
        collectionresourceid: optional if missing create a collection,
        transactionid: transaction_id if a part of workflow,
        data: [{
            resourceid: resourceid of resource to be edited,
            tileid: tileid of resource to be edited if available,
            action: 'add' or 'remove'
        }]
    }
    """

    def create_new_collection(self, request, transaction_id, project_name):
        collection_graphid = "1b210ef3-b25c-11e9-a037-a4d18cec433a"
        name_node_id = "52aa2007-c450-11e9-b5d4-a4d18cec433a"
        resource = Resource()
        resource.graph_id = collection_graphid
        resource.save(transaction_id=transaction_id)
        collection_resourceinstance_id = str(resource.pk)

        tile = Tile.get_blank_tile(nodeid=name_node_id, resourceid=collection_resourceinstance_id)
        stringDataType = StringDataType()
        # TODO(i18n)
        tile.data[name_node_id] = stringDataType.transform_value_for_tile("Collection for {0}".format(project_name))
        tile.save(request=request, transaction_id=transaction_id, index=False)

        collection_resource = Resource.objects.get(pk=resource.pk)

        return collection_resource, tile.tileid

    def add_collection_to_project(self, request, resourceinstaneid, r_resourceinstaneid, transaction_id):
        used_set_node_id = "cc5d6df3-d477-11e9-9f59-a4d18cec433a"  # Project nodegroup (hidden nodegroup)
        related_resource_template["resourceId"] = r_resourceinstaneid
        related_resource_template["ontologyProperty"] = "49c40bf4-e797-4d36-8d54-072cf1fada88"
        related_resource_template["inverseOntologyProperty"] = "c252f48a-3529-4a7a-b873-fed4e9259e1f"
        tile = Tile.get_blank_tile(nodeid=used_set_node_id, resourceid=resourceinstaneid)
        tile.data[used_set_node_id] = [related_resource_template]
        tile.save(request=request, transaction_id=transaction_id, index=False)

        return (tile.tileid, Resource.objects.get(pk=tile.resourceinstance_id))

    def post(self, request):
        def bulk_update(resources):
            """
            Saves and indexes a list of resources

            Arguments:
            resources -- a list of resource models

            Keyword Arguments:
            transaction_id -- a uuid identifing the save of these instances as belonging to a collective load or process

            """

            datatype_factory = DataTypeFactory()
            node_datatypes = {str(nodeid): datatype for nodeid, datatype in Node.objects.values_list("nodeid", "datatype")}
            tiles = []
            documents = []
            term_list = []

            for resource in resources:
                resource.tiles = resource.get_flattened_tiles()
                tiles.extend(resource.tiles)

            # need to save the models first before getting the documents for index
            for tile in tiles:
                TileModel.objects.update_or_create(tile)

            for resource in resources:
                document, terms = resource.get_documents_to_index(
                    fetchTiles=True, datatype_factory=datatype_factory, node_datatypes=node_datatypes
                )

                documents.append(se.create_bulk_item(index=RESOURCES_INDEX, id=document["resourceinstanceid"], data=document))

                for term in terms:
                    term_list.append(se.create_bulk_item(index=TERMS_INDEX, id=term["_id"], data=term["_source"]))

            se.bulk_index(documents)
            se.bulk_index(term_list)

        member_of_set_node_id = "63e49254-c444-11e9-afbe-a4d18cec433a"  # Physical Thing nodegroup
        project_resourceid = request.POST.get("projectresourceid", None)
        collection_resourceid = request.POST.get("collectionresourceid", None)
        data = JSONDeserializer().deserialize(request.POST.get("data"))
        transaction_id = request.POST.get("transactionid", None)
        project_name = Resource.objects.get(pk=project_resourceid).name
        ret = {}
        resources = []
        with transaction.atomic():

            if collection_resourceid is None:
                collection_resource, collection_name_tile_id = self.create_new_collection(request, transaction_id, project_name)
                collection_resourceid = str(collection_resource.resourceinstanceid)
                resources.append(collection_resource)
                (project_used_set_tile_id, resource) = self.add_collection_to_project(
                    request, project_resourceid, collection_resourceid, transaction_id
                )
                resources.append(resource)
                ret = {
                    "collectionResourceid": collection_resourceid,
                    "collectionNameTileId": collection_name_tile_id,
                    "projectUsedSetTileId": project_used_set_tile_id,
                }

            try:
                for datum in data:
                    action = datum["action"]
                    resource_id = datum["resourceid"] if "resourceid" in datum else None
                    tile_id = datum["tileid"] if "tileid" in datum else None

                    if tile_id is not None:
                        tile = Tile.objects.get(pk=tile_id)
                    else:
                        try:
                            tile = Tile.objects.get(resourceinstance=resource_id, nodegroup_id=member_of_set_node_id)
                        except ObjectDoesNotExist as e:
                            tile = Tile.get_blank_tile(nodeid=member_of_set_node_id, resourceid=resource_id)
                            tile.data[member_of_set_node_id] = []

                    list_of_rr_resources = [data["resourceId"] for data in tile.data[member_of_set_node_id]]

                    if collection_resourceid not in list_of_rr_resources and action == "add":
                        related_resource_template["resourceId"] = collection_resourceid
                        related_resource_template["ontologyProperty"] = "31327077-8af5-4398-bbcc-e75675a9d37e"
                        related_resource_template["inverseOntologyProperty"] = "6e7cf6a4-aba0-4a17-9a36-c69412212699"
                        tile.data[member_of_set_node_id].append(related_resource_template)
                        resources.append(Resource.objects.get(pk=tile.resourceinstance_id))
                        tile.save(request=request, transaction_id=transaction_id, index=False)
                    elif collection_resourceid in list_of_rr_resources and action == "remove":
                        rr_data = tile.data[member_of_set_node_id]
                        tile.data[member_of_set_node_id] = [rr for rr in rr_data if rr["resourceId"] != collection_resourceid]
                        resources.append(Resource.objects.get(pk=tile.resourceinstance_id))
                        tile.save(request=request, transaction_id=transaction_id, index=False)
                bulk_update(resources)

                return JSONResponse({"result": ret})

            except Exception as e:
                logger.exception(e)
                response = {"result": "failed", "message": [_("Request Failed"), _("Unable to save")]}
                return JSONResponse(response, status=500)



from storages.backends.s3boto3 import S3Boto3Storage
from django.core.files.storage import Storage


class S3BackendReadOnlyStorage(S3Boto3Storage):
    def generate_filename(self, filename):
        return filename

    def save(self, name, content, max_length=None):
        if content.field_name.endswith("_preloaded"):
            return name
        else:
            return super().save(name, content, max_length)



"""
ARCHES - a program developed to inventory and manage immovable cultural heritage.
Copyright (C) 2013 J. Paul Getty Trust and World Monuments Fund

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

from django.utils.translation import get_language, get_language_bidi
from arches.app.models.resource import Resource
from arches.app.models.tile import Tile
from arches_for_science.models import ManifestXDigitalResource, CanvasXDigitalResource, ManifestXCanvas


def build_string_object(string):
    return {get_language(): {"value": string, "direction": "rtl" if get_language_bidi() else "ltr"}}


def create_manifest_x_digitalresource(manifest, digital_resource):
    manifest_x_digitalresource = ManifestXDigitalResource.objects.create(manifest=manifest, digitalresource=digital_resource)
    return manifest_x_digitalresource


def create_canvas_x_digitalresource(canvas, digital_resource):
    canvas_x_digitalresource = CanvasXDigitalResource.objects.create(canvas=canvas, digitalresource=digital_resource)
    return canvas_x_digitalresource


def create_manifest_x_canvas(manifest, canvas):
    manifest_x_canvas = ManifestXCanvas.objects.create(manifest=manifest, canvas=canvas)
    return manifest_x_canvas


def add_tiles(
    resource_id,
    name=None,
    statement=None,
    id=None,
    type=None,
    service={},
    service_identifiers=[],
    transactionid=None,
    manifest_data=None,
    iiif_type=None,
):
    name_content_node_id = "d2fdc2fa-ca7a-11e9-8ffb-a4d18cec433a"
    identifier_content_node_id = "db05c421-ca7a-11e9-bd7a-a4d18cec433a"
    identifier_type_node_id = "db05c05e-ca7a-11e9-8824-a4d18cec433a"
    type_node_id = "09c1778a-ca7b-11e9-860b-a4d18cec433a"
    statement_content_node_id = "da1fbca1-ca7a-11e9-8256-a4d18cec433a"
    service_type_node_id = "5ceedd21-ca7c-11e9-a60f-a4d18cec433a"
    service_type_conformance_node_id = "cec360bd-ca7f-11e9-9ab7-a4d18cec433a"
    service_identifier_type_node_id = "56f8e759-ca7c-11e9-bda1-a4d18cec433a"
    service_identifier_content_node_id = "56f8e9bd-ca7c-11e9-b578-a4d18cec433a"
    part_of_manifest_node_id = "1b3d7f38-ca7b-11e9-ab2c-a4d18cec433a"

    name_nodegroupid = "d2fdae3d-ca7a-11e9-ad84-a4d18cec433a"
    identifier_nodegroupid = "db05b5ca-ca7a-11e9-82ca-a4d18cec433a"
    type_nodegroupid = "09c1778a-ca7b-11e9-860b-a4d18cec433a"
    statement_nodegroupid = "da1fac57-ca7a-11e9-86a3-a4d18cec433a"
    service_nodegroupid = "29c8c76e-ca7c-11e9-9e11-a4d18cec433a"
    service_identifier_nodegroupid = "56f8e26e-ca7c-11e9-9aa3-a4d18cec433a"

    if name:
        name_count = Tile.objects.filter(nodegroup_id=name_nodegroupid, resourceinstance_id=resource_id).count()
        if name_count == 1:
            name_tile = Tile.objects.get(nodegroup_id=name_nodegroupid, resourceinstance_id=resource_id)
        elif name_count == 0:
            name_tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=name_nodegroupid, resourceid=resource_id)
        else:
            return
        name_tile.data[name_content_node_id] = build_string_object(name)
        name_tile.save(transaction_id=transactionid, index=True)

    if statement:
        statement_count = Tile.objects.filter(nodegroup_id=statement_nodegroupid, resourceinstance_id=resource_id).count()
        if statement_count == 1:
            statement_tile = Tile.objects.get(nodegroup_id=statement_nodegroupid, resourceinstance_id=resource_id)
        elif statement_count == 0:
            statement_tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=statement_nodegroupid, resourceid=resource_id)
        else:
            return
        statement_tile.data[statement_content_node_id] = build_string_object(statement)
        statement_tile.save(transaction_id=transactionid, index=True)

    if service:
        for service_type_conformance, service_type in service.items():
            service_tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=service_nodegroupid, resourceid=resource_id)
            service_tile.data[service_type_conformance_node_id] = build_string_object(service_type_conformance)
            service_tile.data[service_type_node_id] = service_type
            service_tile.save(transaction_id=transactionid, index=False)

    if service_identifiers:
        for service_identifier in service_identifiers:
            for service_identifier_content, service_identifier_type in service_identifier.items():
                service_identifier_tile = Tile.get_blank_tile_from_nodegroup_id(
                    nodegroup_id=service_identifier_nodegroupid, resourceid=resource_id, parenttile=service_tile
                )
                service_identifier_tile.data[service_identifier_type_node_id] = service_identifier_type
                service_identifier_tile.data[service_identifier_content_node_id] = build_string_object(service_identifier_content)
                service_identifier_tile.save(transaction_id=transactionid, index=False)

    if id:
        for identifier_content, identifier_type in id.items():
            identifier_tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=identifier_nodegroupid, resourceid=resource_id)
            identifier_tile.data[identifier_content_node_id] = build_string_object(identifier_content)
            identifier_tile.data[identifier_type_node_id] = identifier_type
            identifier_tile.save(transaction_id=transactionid, index=False)

    if type:
        type_tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=type_nodegroupid, resourceid=resource_id)
        type_tile.data[type_node_id] = type
        type_tile.save(transaction_id=transactionid, index=True)

    if iiif_type == "canvas":
        manifest_digital_resource = ManifestXDigitalResource.objects.get(manifest=manifest_data["@id"])
        part_of_tile = Tile.get_blank_tile_from_nodegroup_id(nodegroup_id=part_of_manifest_node_id, resourceid=resource_id)
        part_of_tile.data[part_of_manifest_node_id] = [
            {
                "resourceId": manifest_digital_resource.digitalresource,
                "ontologyProperty": "c04138e2-7ea8-4ca0-9e81-d4355362b686",
                "resourceXresourceId": "",
                "inverseOntologyProperty": "c07f05bc-0397-49c8-880a-46ed69ce2a4a",
            }
        ]
        part_of_tile.save(transaction_id=transactionid, index=True)


def create_digital_resource(instance, iiif_type, canvas=None):
    """Creates the digital resources resource instance representing manifest
    and also creates the manifest_x_canvas record
    """
    iiif_manifest_valueid = ["305c62f0-7e3d-4d52-a210-b451491e6100"]
    internal_id_valueid = ["768b2f11-26e4-4ada-a699-7a8d3fe9fe5a"]
    web_service_valueid = ["e208df66-9e61-498b-8071-3024aa7bed30"]
    digital_resource_graph = "707cbd78-ca7a-11e9-990b-a4d18cec433a"

    manifest_data = instance.manifest
    service = {manifest_data["@context"]: web_service_valueid}  # TODO canvas does not have its own service
    type = iiif_manifest_valueid  # TODO canvas type need to be added
    transactionid = instance.transactionid

    resource = Resource(graph_id=digital_resource_graph)
    resource.save(transaction_id=instance.transactionid)
    resource_id = resource.pk

    if iiif_type == "manifest":
        name = manifest_data["label"]
        statement = manifest_data["description"]
        id = {str(instance.globalid): internal_id_valueid}
        service_identifiers = [
            {manifest_data["@id"]: ["f32d0944-4229-4792-a33c-aadc2b181dc7"]},
        ]
    elif iiif_type == "canvas":
        name = canvas["label"]
        statement = None
        id = {canvas["images"][0]["resource"]["service"]["@id"]: internal_id_valueid}
        service_identifiers = [
            {canvas["images"][0]["resource"]["@id"]: ["f32d0944-4229-4792-a33c-aadc2b181dc7"]},
            {canvas["images"][0]["resource"]["service"]["@id"]: ["768b2f11-26e4-4ada-a699-7a8d3fe9fe5a"]},
            {canvas["images"][0]["@id"]: ["768b2f11-26e4-4ada-a699-7a8d3fe9fe5a"]},
        ]
    add_tiles(
        resource_id=resource_id,
        name=name,
        statement=statement,
        id=id,
        type=type,
        service=service,
        service_identifiers=service_identifiers,
        transactionid=transactionid,
        manifest_data=manifest_data,
        iiif_type=iiif_type,
    )
    return resource_id


def update_manifest_digital_resource(instance):
    manifest_data = instance.manifest
    manifest_resource_id = ManifestXDigitalResource.objects.get(manifest=manifest_data["@id"]).digitalresource
    add_tiles(
        manifest_resource_id, name=manifest_data["label"], statement=manifest_data["description"], transactionid=instance.transactionid
    )
    return manifest_resource_id


def digital_resources_for_manifest(instance, created):
    """the main function to create/update the digital resource for the manifest"""
    # the creation of the resource will be only applied to the local manifests that can be created and updated

    if created:
        manifest_resource_id = create_digital_resource(instance, "manifest")
        create_manifest_x_digitalresource(instance.manifest["@id"], manifest_resource_id)

    else:
        if ManifestXDigitalResource.objects.filter(manifest=instance.manifest["@id"]).count() == 1:
            update_manifest_digital_resource(instance)
        else:
            manifest_resource_id = create_digital_resource(instance, "manifest")
            create_manifest_x_digitalresource(instance.manifest["@id"], manifest_resource_id)


def digital_resources_for_canvases(instance):
    """the main function to create/update the digital resource for the canvases"""
    manifest_data = instance.manifest

    # add the canvas record to canvas_x_digitalresource if not already available
    for canvas in manifest_data["sequences"][0]["canvases"]:
        if not CanvasXDigitalResource.objects.filter(canvas=canvas["images"][0]["resource"]["service"]["@id"]).exists():
            canvas_resource_id = create_digital_resource(instance, "canvas", canvas)
            create_canvas_x_digitalresource(canvas["images"][0]["resource"]["service"]["@id"], canvas_resource_id)

        if not ManifestXCanvas.objects.filter(
            manifest=manifest_data["@id"], canvas=canvas["images"][0]["resource"]["service"]["@id"]
        ).exists():
            create_manifest_x_canvas(manifest_data["@id"], canvas["images"][0]["resource"]["service"]["@id"])

    # remove the canvas record in manifest_x_canvas that was removed from the current manifest
    current_canvases = [canvas["images"][0]["resource"]["service"]["@id"] for canvas in manifest_data["sequences"][0]["canvases"]]
    for manifest_x_canvas in ManifestXCanvas.objects.filter(manifest=manifest_data["@id"]):
        if manifest_x_canvas.canvas not in current_canvases:
            manifest_x_canvas.delete()
